/*******************************************************************************
*
* File:         p6a_demo.c
* Description:  系统测试文件
* Version:      V0.1
* Date:         2020-02-22
* Author:       Tempest
 *******************************************************************************/
#include "p6a_demo.h"
#include "ArmCM0.h"
#include "tau_device_datatype.h"
#include "tau_log.h"
#include "tau_operations.h"
#include "tau_common.h"
#include "tau_delay.h"
#include "hal_dsi_rx_ctrl.h"
#include "hal_dsi_tx_ctrl.h"
#include "hal_swire.h"
#include "hal_timer.h"
#include "hal_system.h"
#include "hal_gpio.h"
#include "test_cfg_global.h"
#include "hal_pwr.h"
#include "hal_pwm.h"
#include <stdio.h>

#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "p6a"

#if _DEMO_GOOGLE_P6A_EN

/******************** FEATURE开关 ********************/
#define TOUCH_ENABLE                   false    /* Touch转换开关 */
#define RX_START_WITHOUT_RST           true    /* 不等待AP RESET直接启动RX,仅作为调试使用 */
#define RX_WAIT_TEAR_ON                false     /* 等待AP_TEAR_ON */
#define RX_RESOLUTION_CHANGE_ENABLE    true     /* 支持分辨率切换开关, AP存在分辨率切换时需要打开 */
#define RX_READ_HW_ACK                 false    /* AP DCS读命令使用硬件回复 */
#define TX_START_AFTER_APRST           false    /* 等待AP_reset 后做Panel初始化, 用于热拔插电源不稳定导致初始化失败 */
#define TX_USE_CMD_MODE                false    /* command mode输出开关 配置为True时需要把TD TE与 AP TE 接一起*/
#define TX_CMD_MODE_WITHOUT_TE         false    /* 屏端TE直接输出给AP,Scaler不看TE信号,AP每输入一帧输出一帧(C2C 60Hz->60Hz) */
#define AP_SWIRE_OUTPUT                false     /* swire信号输出,OLED项目配置 */
#define ANALOG_PWM_OUTPUT              false     /* 模拟PWM 调光开关 */
#define SHARE_FLASH_ENABLE             false    /* 共享flash开关 */
/*****************************************************/

/******************** 输出屏幕选择 ********************/
#define AMOLED_ICNA3511                1                   /* 4lane FHD Panel */
/*****************************************************/

/****************** 系统相关参数配置 ******************/
#define MAIN_POWER_SELECT              PWR_SEL_VCC           /* 主供电电源选择 */
#define SLEEP_MODE_POWER               PWR_SLEEP_IN_IOV18 //PWR_SLEEP_IN_TP18     /* 息屏电源选择 */
#define SLEEP_MODE_SELECT              PWR_NORMAL_SLEEP_MODE //PWR_DEEP_SLEEP_MODE //PWR_NORMAL_SLEEP_MODE //   /* sleep mode 配置 */
#define SWIRE_DEFAULT_PULSE            37 // 31                    /* SWIRE 波形配置 */ 2.6
#define PWM_FREQUENCY                  30000                 /* PWM输出频率30Khz */
#define PWM_DUTY_STEP                  255                   /* PWM调光阶数255阶 */
/*****************************************************/

/********************RX 基本参数配置*******************/
//AP MIPI数据信息
/* 输入分辨率 */
#define INPUT_WIDTH  1080
#define INPUT_HEIGHT 2400
/* 输入 MIPI lane rate,需要正确配置,可50M step调整 */
#define INPUT_MIPI_LANE_RATE 1450000000
/* 输入图像格式 */
#define INPUT_COLOR_MODE DSI_RGB888
/* 输出数据格式(DSI_DATA_CMD_MODE , DSI_DATA_VIDEO_MODE) */
#define INPUT_DATA_MODE DSI_DATA_CMD_MODE
/* 输入mipi lane数量(DSI_RX_LANE_x     x为1-4) */
#define INPUT_MIPI_LANE_NUM DSI_LANE_4
/* 输入为video mode 时数据格式 */
#define INPUT_VIDEO_MODEL DSI_NONBURST_EVENT
/* 输入虚拟通道(0-3) */
#define INPUT_VC DSI_VC_0
/* 输入的帧率(60/90/120/144Hz) */
#define INPUT_FRAME_RATE DSI_FRAME_RATE_60HZ
/* 输入数据是否DSC压缩 */
#define INPUT_COMPRESS true
/*****************************************************/

/********************TX 基本参数配置*******************/
#if AMOLED_ICNA3511
//输出MIPI数据信息
/* 输出分辨率配置 */
#define OUTPUT_WIDTH     1044
#define OUTPUT_HEIGHT    2260
/* 输出虚拟通道(0-3) */
#define OUTPUT_VC DSI_VC_0
/* 输出mipi lane数量(DSI_RX_LANE_x     x为1-4) */
#define OUTPUT_LANE_NUMBER DSI_LANE_4
/* 输出数据格式(DSI_DATA_CMD_MODE , DSI_DATA_VIDEO_MODE) */
#if TX_USE_CMD_MODE
#define OUTPUT_DATA_MODE DSI_DATA_CMD_MODE
#else
#define OUTPUT_DATA_MODE DSI_DATA_VIDEO_MODE
#endif
/* 输出为video mode时的数据格式 */
#define OUTPUT_VIDEO_MODEL DSI_BURST_MODE
/* 输出 VSA */
#define OUTPUT_VSA  2  //2
/* 输出 VBP */
#define OUTPUT_VBP  8   //8
/* 输出 VBP */
#define OUTPUT_VFP  62 //62
/* 输出 VSA */
#define OUTPUT_HSA  2   //2
/* 输出 HBP */
#define OUTPUT_HBP  28
/* 输出 HFP */
#define OUTPUT_HFP  168  //168
/* 初始化模式命令传输类型 LP/HS */
#define TX_INIT_TYPE            DSI_CMD_TX_LP
#endif
/******************************************************/

#if TOUCH_ENABLE
#include "app_tp_transfer.h"
#endif

/* 全局handle */
static hal_dsi_rx_ctrl_handle_t *g_rx_ctrl_handle = NULL;
static hal_dsi_tx_ctrl_handle_t *g_tx_ctrl_handle = NULL;

/* 屏初始化完成标志位 */
static bool panel_display_done = false;
static bool sg_system_resume = false;
static bool sg_system_suspend = false;
static bool sg_exit_idle_mode_flag = false;

static volatile bool g_resolution_change = false;
static uint32_t pps_renew_flag = 0;
static uint32_t pwr_rst_flag = 0;

#if RX_WAIT_TEAR_ON
static bool sg_ap_set_tear_on = false;
#endif
#if TX_START_AFTER_APRST
static bool sg_tx_start_in_process = false;
#endif

/* soft swire */
#define AVDD_EN                 ( IO_PIN_15 )
#define SWIRE_EN                ( IO_PIN_17 )
#define SWIRE_MAX_NUM           ( 10 )
#define SWIRE_DEFAULT_NUM       ( 37 )  // 43 对应ELVSS -2.2V  35 对应 ELVSS -3V
#define SOFT_SWIRE_TIMER        ( TIMER_NUM3 )
uint8_t s_swire_pwm_counter     = 0;

uint16_t rd_51_val,rd_51_val2;

/* AP reset 回调函数声明 */
static void ap_rstn_pull_high_cb(void *data);
static void ap_rstn_pull_down_cb(void *data);
static void app_mipi_rx_start_cb(void *data);
static void soft_swire_init(void);

/*************************DCS 命令处理函数 BEGIN*************************/
#if RX_READ_HW_ACK
/**
* @brief  配置AP硬件回读
* @param  none
* @retval none
*/
static void app_set_dcs_hw_ack()
{
    hal_dsi_rx_ctrl_set_auto_ack(g_rx_ctrl_handle, HAL_RX_QRESP_CODE0, DSI_ACK_DT_DCS_SHORT_RESPONSE_1B, 0xFA, 1, 0x00);
}
#endif
/**
* @brief  ap 读回调函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_read(uint8_t data_type, uint8_t dcs_cmd, uint8_t param)
{
 	if (dcs_cmd == 0x04)
	{
		hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
									 DSI_ACK_DT_DCS_LONG_RESPONSE,
									 DSI_VC_0,
									 3, 0x8A,0x80,0x00);
	}
	else if (dcs_cmd == 0xa1)
	{
		hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
									 DSI_ACK_DT_DCS_LONG_RESPONSE,
									 DSI_VC_0,
									 13, 0x0C,0x12,0x0C,0xBF,0x00,0x00,0xB7,0x1A,0x14,0x38,0x11,0x20,0x21);
	}
	else if (dcs_cmd == 0xDA)
	{
		hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
									 DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
									 DSI_VC_0,
									 1, 0x8A);
	}
	else if (dcs_cmd == 0xDB)
	{
		hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
									 DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
									 DSI_VC_0,
									 1, 0x80);
	}
	else if (dcs_cmd == 0xDC)
	{
		hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
									 DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
									 DSI_VC_0,
									 1, 0x00);
	}
	else if (dcs_cmd == 0xD8)
	{
		hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
									 DSI_ACK_DT_DCS_LONG_RESPONSE,
									 DSI_VC_0,
									 5, 0x0C,0xAC,0x84,0xBE,0xFB);
	}
	else
	{
		uint32_t return_size = hal_dsi_rx_ctrl_get_max_ret_size(g_rx_ctrl_handle);
		hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
									 DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
									 DSI_VC_0,
									 1, 0);
		TAU_LOGD("r[%x] [%d] err!!!!!!\n", dcs_cmd, return_size);
	}
	
//	TAU_LOGD("r %x\n",dcs_cmd);
	return true;	
}

uint8_t pps[128] = {0x11,0x00,0x00,0x89,0x30,0x80,0x09,0x60,0x04,0x38,0x00,0x30,0x02,0x1C,0x02,0x1C,
                        0x02,0x00,0x02,0x0E,0x00,0x20,0x04,0xA6,0x00,0x07,0x00,0x0C,0x02,0x0B,0x02,0x1F,
                        0x18,0x00,0x10,0xF0,0x03,0x0C,0x20,0x00,0x06,0x0B,0x0B,0x33,0x0E,0x1C,0x2A,0x38,
                        0x46,0x54,0x62,0x69,0x70,0x77,0x79,0x7B,0x7D,0x7E,0x01,0x02,0x01,0x00,0x09,0x40,
                        0x09,0xBE,0x19,0xFC,0x19,0xFA,0x19,0xF8,0x1A,0x38,0x1A,0x78,0x1A,0xB6,0x2A,0xF6,
                        0x2B,0x34,0x2B,0x74,0x3B,0x74,0x6B,0xF4,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};


#if RX_RESOLUTION_CHANGE_ENABLE
/* PPS update callback 用于分辨率切换case */
static bool pps_update_handle(uint8_t *pps, uint8_t size, uint32_t pic_width, uint32_t pic_height)
{
	
    /* AVDD 上电, 用于解决息屏开屏PPS不更新问题 */
 //   hal_gpio_set_output_data(IO_PAD_AP_PWMEN, IO_LVL_HIGH);
    if (pic_width != g_rx_ctrl_handle->base_info.src_w || pic_height != g_rx_ctrl_handle->base_info.src_h)
    {
        hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_USER_MODE);
        /* PPS Update 且分辨率发生变化 */
        g_rx_ctrl_handle->base_info.src_w = pic_width;
        g_rx_ctrl_handle->base_info.src_h = pic_height;
        /* 注意部分基板更新PPS前不发 Compression Mode Command的情况 */
        g_rx_ctrl_handle->compress_en = true;  
		if(pic_width > 720)
		{
			g_tx_ctrl_handle->base_info.src_w = pic_width;
			g_tx_ctrl_handle->base_info.src_h = pic_height;
		}
//		hal_dsi_rx_ctrl_pre_init_pps(g_rx_ctrl_handle, pps, 128);
		hal_dsi_rx_ctrl_toggle_resolution(g_rx_ctrl_handle);
    }
	    hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_60HZ_MODE);
    //TAU_LOGD("PPS Update[%d][%d] [%d][%d]\n", pic_width, pic_height, g_rx_ctrl_handle->base_info.src_w, g_rx_ctrl_handle->base_info.src_h);
    return true;
}
#endif



/**
* @brief  ap display on处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_display_on(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
//    TAU_LOGD("disp on");
//	if (start_display_on == false){
		hal_dsi_tx_ctrl_write_cmd(0x05, 0, 2, 0x29);
//	}
    TAU_LOGD("disp on \n");
    return true;
}


/**
* @brief  ap display off处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_display_off(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
#if ANALOG_PWM_OUTPUT
    hal_pwm_enable(false);
#endif

    TAU_LOGD("disp off %d\n", panel_display_done);
    return true;
}


/*****************************************************************************
*GPIO发送swire波形
*flag:  =0, SWIRE=0;  =1,仅发送SWIRE信号; =2, 先置高再发SWIRE信号
*num: 发几个脉冲
*注意FLAG=1时无GPIO初始化!!!!!!
*****************************************************************************/
void Gpio_swire_output(uint8_t flag, uint8_t num)
{
	uint8_t ii;

	if (flag)
	{
		if (flag ==2)
		{
			hal_gpio_init_output(IO_PAD_AP_SWIRE, IO_LVL_HIGH);	  
			delayMs(2);
		}
		for (ii =0; ii< num; ii++)
		{
			hal_gpio_set_output_data(IO_PAD_AP_SWIRE, IO_LVL_LOW);
			delayUs(10);
			hal_gpio_set_output_data(IO_PAD_AP_SWIRE, IO_LVL_HIGH);
			delayUs(9);
		}
	}
	else
	{
		hal_gpio_init_output(IO_PAD_AP_SWIRE, IO_LVL_LOW);	  
	}
}



/**
* @brief  ap enter sleep mode处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_enter_sleep_mode(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{	
#if 1
//	delayMs(12); 
	hal_dsi_tx_ctrl_write_cmd(0x05, 0, 1, 0x28);
	delayMs(80);  //40
//	hal_dsi_tx_ctrl_write_cmd(0x05, 0, 1, 0x10);	
//	delayMs(80); //80
	hal_gpio_init_output(SWIRE_EN, IO_LVL_LOW);
	hal_timer_deinit(SOFT_SWIRE_TIMER);
	hal_timer_stop(SOFT_SWIRE_TIMER);	
	    /* FIXME set tear on*/
    hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_STOP_MODE);
	delayMs(41);
	hal_dsi_tx_ctrl_write_cmd(0x05, 0, 1, 0x10);
    delayMs(20);	
	    /* AVDD_EN close*/
	 hal_gpio_init_output( AVDD_EN, IO_LVL_LOW ); //  SWIRE close 
#if AP_SWIRE_OUTPUT
    /* Swire close */
   hal_swire_enable(false);
#endif
#endif
    /* Wait AP reset down*/
 //   hal_gpio_set_ap_reset_int(ENABLE, ap_rstn_pull_down_cb, DETECT_LOW_LVL);
// TAU_LOGD("enter sleep mode\n");
// delayMs(500);
    return true;
}

/**
* @brief  ap exit sleep mode处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_exit_sleep_mode(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
//#if AP_SWIRE_OUTPUT
//    /* AVDD 上电, 用于解决息屏开屏PPS不更新问题 */
    hal_gpio_set_output_data(AVDD_EN, IO_LVL_HIGH);
//#endif

#if TX_START_AFTER_APRST
    if (panel_display_done == false)
    {
        sg_tx_start_in_process = true;
    }
#endif

    TAU_LOGD("exit sleep mode \n");

    return true;
}



/**
* @brief  调光处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/

uint8_t value_51H,value_51L;
static bool reg53_E8_fg=0;
static bool ap_dcs_set_backlight(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{	
	//手机端0x04~7FF(2043) 映射0xFFF
#if 0
	value_51H = dcs_packet->packet_param[0];
	value_51L = dcs_packet->packet_param[1];
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0x51, value_51H, value_51L);
#else
		rd_51_val = dcs_packet->packet_param[0];
	rd_51_val <<=8;
	rd_51_val |= dcs_packet->packet_param[1];

//	rd_51_val2 = (rd_51_val-0x04)*4091/2043+0x04;   //0xFFF max
	if(rd_51_val == 0x00) {
	
	  rd_51_val2 = 0x04; 
	
	}
	else if((rd_51_val >0x00)&&(rd_51_val <= 0x7FF) ){
	
	  rd_51_val2 = (rd_51_val-0x04)*4091/2043+0x04;   //0xFFF max
	
	}
	else if(rd_51_val >=0x7FF) {
	
	  rd_51_val2 = 0xFFF; 
	
	}
		hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0x51, rd_51_val2>>8, rd_51_val2&0x00FF);
#endif
//	 TAU_LOGD("AP_51[0x%04X],IC_51[0x%04X]\n",rd_51_val,rd_51_val2);	
    return true;
}

#if RX_WAIT_TEAR_ON
/**
* @brief  ap set tear on 处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_tear_on(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
    TAU_LOGD("ap_set_tear_on\n");
    sg_ap_set_tear_on = true;
    if (panel_display_done == false)
    {
        TAU_LOGD("gen a tear\n");
        hal_dsi_tx_ctrl_gen_a_tear_signal();
    }
    else
    {
        hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_60HZ_MODE);
    }
    return true;
}

/**
* @brief  ap set tear off 处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_tear_off(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
    TAU_LOGD("ap_set_tear_off \n");
    sg_ap_set_tear_on = false;
    hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_USER_MODE);
    return true;
}
#endif

/**
* @brief  护眼模式回调函数
* @param  rx_ctrl_handle: dsi rx handle;
*         dcs packet: dcs_packet
* @retval true/false
*/
static bool ap_dcs_set_exit_idle_mode(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
    sg_exit_idle_mode_flag = true;
    hal_dsi_rx_ctrl_set_hw_cmd_filter(g_rx_ctrl_handle, HAL_RX_DCS_FILTER_B, 0x2C, 0x2C);
    TAU_LOGD("exit idle mode,skip 0x2C\n");
    return true;
}

#define TE_TIMER    TIMER_NUM2
static void soft_te_timer_cb(void *data)
{
    /*
        S8 的屏接的是TP1.8V, AC 启动后需要等到TP1.8 起来后再初始化屏, 所以在TP 起来前需要通过软件产生TE给手机，避免手机卡死
    */
//    if (panel_display_done == false)
//    {
        hal_dsi_tx_ctrl_gen_a_tear_signal();
        hal_timer_start(TE_TIMER, 16, soft_te_timer_cb, NULL);
//    }
//    else
//    {
//        hal_dsi_rx_ctrl_set_hw_tear_mode(g_rx_ctrl_handle);
//    }
}

static void soft_te_timer_init()
{
//    TAU_LOGD("soft_te_timer_init");
    hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle,TE_USER_MODE);
    hal_timer_init(TE_TIMER);
    hal_timer_start(TE_TIMER, 1, soft_te_timer_cb, NULL);
}

/**
* @brief  帧率切换处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_frame_change(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
    if (dcs_packet->param_length == 1)
    {
        if (dcs_packet->packet_param[0] == 0x01)
        {
		//	hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle,TE_USER_MODE);
            hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_60HZ_MODE);
        }
        else 
        {
		//	soft_te_timer_init();
           hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_120HZ_MODE);
        }
    }
//    TAU_LOGD("frame_change %x ,size %d, data %d\n", dcs_packet->dcs_command, dcs_packet->param_length, dcs_packet->packet_param[0]);
    return true;
}

static bool ap_set_FPS_53(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
	uint8_t value_b1 =0;	
	
    value_b1 =  dcs_packet->packet_param[0]; 

	if(value_b1 == 0x38)  // AP FPS ON
	{
    if(rd_51_val2>=0x88F){ //DFF
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x11, 0x8F,0xFF);   // DDIC FPS ON
	}
//	else if((rd_51_val >0xBFF)&&(rd_51_val <= 0xDFF) ){
//         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x11, 0x8E,0xAF);   // DDIC FPS ON
//	}
//	 else if((rd_51_val >0xA8F)&&(rd_51_val <= 0xBFF) ){
//         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x11, 0x8C,0xFF);   // DDIC FPS ON
//	}
//	  else if((rd_51_val >0x88F)&&(rd_51_val <= 0xA8F) ){
//         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x11, 0x8B,0xFF);   // DDIC FPS ON
//	}
	  else if((rd_51_val >0x6FF)&&(rd_51_val <= 0x88F) ){
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x11, 0x89,0x8F);   // DDIC FPS ON
	}
	 else if((rd_51_val >0x4FF)&&(rd_51_val <= 0x6FF) ){
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x11, 0x86,0xFF);   // DDIC FPS ON
	}
	 else{
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97,0x11, 0x83,0xFF);   // DDIC FPS ON
	}
	}
	else if(value_b1 == 0x28)  // AP FPS OFF
	{

  //       hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x00, (rd_51_val2>>8), rd_51_val2&0xFF);// DDIC FPS OFF
    if(rd_51_val2>=0xDFF){
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x00, 0x0F,0xFF);   // DDIC FPS OFF
	}
	else if((rd_51_val >0xBFF)&&(rd_51_val <= 0xDFF) ){
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x00, 0x0D,0x8F);  
	}
	 else if((rd_51_val >0xA8F)&&(rd_51_val <= 0xBFF) ){
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x00, 0x0B,0x8F);  
	}
	  else if((rd_51_val >0x88F)&&(rd_51_val <= 0xA8F) ){
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x00, 0x09,0xFF);  
	}
	  else if((rd_51_val >0x6FF)&&(rd_51_val <= 0x88F) ){
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x00, 0x07,0x8F);  
	}
	 else if((rd_51_val >0x4FF)&&(rd_51_val <= 0x6FF) ){
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97, 0x00, 0x05,0xFF);  
	}
	 else{
         hal_dsi_tx_ctrl_write_cmd( 0x39, 0, 4, 0x97,0x00, 0x02,0xFF);   
	}
        
	}

//   	TAU_LOGD("REG53[%x]", value_b1);
    return true;
}


/*************************DCS 命令处理函数 END*************************/

/* 客制化DCS command 处理函数表格 */
static const hal_dcs_execute_entry_t g_cus_rx_dcs_execute_table[] =
{
    {DCS_SET_DISPLAY_ON, ap_dcs_set_display_on, false},
    {DCS_SET_DISPLAY_OFF, ap_dcs_set_display_off, true},
    {0x51, ap_dcs_set_backlight, true},
	{0x53, ap_set_FPS_53, true},  //B1
    {DCS_ENTER_SLEEP_MODE, ap_dcs_set_enter_sleep_mode, true},
    {DCS_EXIT_SLEEP_MODE, ap_dcs_set_exit_sleep_mode, true},
//   {0x60, ap_dcs_set_frame_change, true},
#if RX_WAIT_TEAR_ON
    {DCS_SET_TEAR_ON, ap_dcs_set_tear_on, true},
    {DCS_SET_TEAR_OFF, ap_dcs_set_tear_off, true},
#endif
  //  {0x38, ap_dcs_set_exit_idle_mode, true},
    {0, NULL, false}                        //{0,NULL,false} 数组最后一个固定成员，作为table结尾的判断标准
};

/**
* @brief  panel reset
* @param  none
* @retval none
*/
static void app_tx_panel_reset(void)
{
#if SHARE_FLASH_ENABLE
    hal_flash_share_mode(true);
#endif
    hal_gpio_set_output_data(IO_PAD_TD_RSTN, IO_LVL_HIGH);
    delayMs(10);
    hal_gpio_set_output_data(IO_PAD_TD_RSTN, IO_LVL_LOW);
    delayMs(10);
    hal_gpio_set_output_data(IO_PAD_TD_RSTN, IO_LVL_HIGH);
//		hal_gpio_init_output(AVDD_EN, IO_LVL_HIGH);	
    delayMs(50);

}

static void send_panel_init_code(uint32_t size, uint8_t* data)
{
    uint32_t data_offeset = 0;
    uint8_t data_type;
    uint8_t vc;
    uint8_t data_size;
    uint8_t* p_data;

    while(data_offeset < size)
    {
        data_type = data[data_offeset];
        vc = data[data_offeset + 1];
        data_size = data[data_offeset + 2];
        p_data = &data[data_offeset + 3];
        hal_dsi_tx_ctrl_write_array_cmd(data_type, vc, data_size, p_data);
        data_offeset = data_offeset + data_size + 3;
        delayUs( 100 );
    }
}

const uint8_t panel_init_code[] =
{
0x39, 0, 3, 0x9C, 0xA5, 0xA5,
0x39, 0, 3, 0xFD, 0x5A, 0x5A,
0x39, 0, 2, 0x9F, 0x0F,
0x39, 0, 2, 0xB3, 0x00,	
0x39, 0, 2, 0x9F, 0x01,
0x39, 0, 4, 0xB2, 0x51, 0x04, 0x69,
0x39, 0, 36, 0xB3, 0x01, 0x6F, 0x00, 0x0A, 0x2E, 0x00, 0x01, 0x6F, 0x00, 0x0A, 0x2E, 0x00, 0x01, 0x6F, 0x00, 0x0A, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x6F, 0x00, 0x0A, 0x14, 0x44, 0x1C, 0x1C, 0x1C, 0x18, 0x1C,
0x39, 0, 3, 0xD9, 0x01, 0x1F,
0x39, 0, 4, 0xCF, 0x01, 0x00, 0x00,
0x39, 0, 2, 0xD0, 0x01,
0x39, 0, 2, 0xD3, 0x00,
0x39, 0, 2, 0xD5, 0x03,
0x39, 0, 2, 0x9F, 0x02,
0x39, 0, 26, 0xB2, 0x00, 0xC1, 0x32, 0x10, 0x11, 0x12, 0x12, 0x00, 0x08, 0x78, 0x21, 0x11, 0x14, 0x13, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x21, 0x21, 0x11, 0x1A,	//power sequence
0x39, 0, 2, 0xB3, 0x0B,
0x39, 0, 14, 0xB4, 0x00, 0x00, 0x8F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  
0x39, 0, 7, 0xB5, 0x00, 0x2F, 0x2F, 0x2F, 0x2F, 0x2F,
0x39, 0, 20, 0xB6, 0x01, 0x00, 0x00, 0x11, 0x11, 0x11, 0x1C, 0x08, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21,
0x39, 0, 17, 0xB7, 0x00, 0x10, 0x15, 0x12, 0x10, 0x15, 0x00, 0x06, 0x03, 0x03, 0x06, 0x03, 0x03, 0x06, 0x03, 0x03,	
0x39, 0, 14, 0xB8, 0x00, 0x00, 0x00, 0x73, 0x2D, 0x73, 0x2D, 0x73, 0x2D, 0x73, 0x2D, 0x73, 0x2D,
0x39, 0, 9, 0xB9, 0x00, 0x00, 0x50, 0x00, 0x00, 0x17, 0x17, 0x00,
0x39, 0, 9, 0xBA, 0x08, 0xFF, 0x36, 0x03, 0x36, 0x03, 0x11, 0x01,
0x39, 0, 26, 0xBB, 0x00, 0xCD, 0x0F, 0xC3, 0x64, 0x00, 0xCD, 0x0F, 0xC3, 0x64, 0x00, 0xCD, 0x0F, 0xC3, 0x64, 0x00, 0xCD, 0x0F, 0xC3, 0x64, 0x00, 0xCD, 0x64, 0x6E, 0x64,
0x39, 0, 6, 0xBE, 0x5F, 0x16, 0x04, 0x5F, 0x75,
0x39, 0, 6, 0xBF, 0x08, 0x00, 0x10, 0x10, 0x08,
0x39, 0, 17, 0xC1, 0x23, 0x00, 0x24, 0x00, 0x05, 0x00, 0x3D, 0x21, 0x00, 0x3D, 0x00, 0x01, 0x00, 0x3D, 0x00, 0x22,
0x39, 0, 17, 0xC2, 0x23, 0x00, 0x24, 0x00, 0x05, 0x00, 0x3D, 0x21, 0x00, 0x3D, 0x00, 0x01, 0x00, 0x3D, 0x00, 0x22,
0x39, 0, 8, 0xC3, 0x11, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 17, 0xC4, 0x00, 0x05, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x1f, 0x00, 0x00, 0x08,
0x39, 0, 27, 0xC5, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 27, 0xC6, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x05, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0x55, 0x00, 0x01, 0x00,
0x39, 0, 14, 0xC7, 0x00, 0x1A, 0x25, 0x24, 0x20, 0x00, 0x24, 0x20, 0x24, 0x20, 0x00, 0x20, 0x20,
0x39, 0, 14, 0xC8, 0x00, 0x35, 0x1A, 0x20, 0x20, 0x00, 0x20, 0x20, 0x20, 0x20, 0x00, 0x00, 0x00,
0x39, 0, 27, 0xC9, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 27, 0xCA, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 27, 0xCB, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 27, 0xCC, 0x04, 0x3A, 0x3B, 0x1A, 0xCD, 0x00, 0x1A, 0xCD, 0x1A, 0xCD, 0x00, 0x44, 0xBF, 0x00, 0x19, 0xBA, 0x20, 0x68, 0x00, 0x24, 0x68, 0x00, 0x00, 0x00, 0x24, 0x6B,
0x39, 0, 25, 0xCD, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 25, 0xCE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 25, 0xCF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 25, 0xD0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 25, 0xD1, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 17, 0xD2, 0x00, 0x00, 0x0F, 0x00, 0x00, 0x09, 0x00, 0x00, 0x09, 0x00, 0x00, 0x09, 0x00, 0x00, 0x09, 0x00,
0x39, 0, 17, 0xD3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 15, 0xD7, 0x44, 0x01, 0x3F, 0x24, 0x56, 0x24, 0x56, 0x01, 0x11, 0x99, 0x0A, 0x00, 0x00, 0x03,
0x39, 0, 26, 0xD8, 0x00, 0x00, 0xC5, 0x9F, 0x40, 0x00, 0x00, 0xC5, 0x9F, 0x40, 0x00, 0x00, 0xC5, 0x9F, 0x40, 0x00, 0x00, 0xC5, 0x9F, 0x40, 0x00, 0x00, 0xC5, 0x00, 0x00,
0x39, 0, 12, 0xD9, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 6, 0xE1, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 26, 0xE2, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 13, 0xE3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 9, 0xE7, 0x80, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x22,
0x39, 0, 12, 0xEE, 0x00, 0x00, 0x00, 0xBB, 0x58, 0x08, 0xA2, 0x50, 0x88, 0x88, 0x29,
0x39, 0, 6, 0xEF, 0x00, 0x80, 0x82, 0x80, 0x85,
0x39, 0, 2, 0x9F, 0x03,
0x39, 0, 33, 0xB2, 0x11, 0x00, 0x00, 0x89, 0x30, 0x80, 0x09, 0x60, 0x04, 0x38, 0x00, 0x0C, 0x04, 0x38, 0x04, 0x38, 0x02, 0x00, 0x03, 0x1C, 0x00, 0x20, 0x01, 0x7E, 0x00, 0x0F, 0x00, 0x0C, 0x08, 0xBB, 0x04, 0x3D,
0x39, 0, 33, 0xB3, 0x18, 0x00, 0x10, 0xF0, 0x03, 0x0C, 0x20, 0x00, 0x06, 0x0B, 0x0B, 0x33, 0x0E, 0x1C, 0x2A, 0x38, 0x46, 0x54, 0x62, 0x69, 0x70, 0x77, 0x79, 0x7B, 0x7D, 0x7E, 0x01, 0x02, 0x01, 0x00, 0x09, 0x40,
0x39, 0, 25, 0xB4, 0x09, 0xBE, 0x19, 0xFC, 0x19, 0xFA, 0x19, 0xF8, 0x1A, 0x38, 0x1A, 0x78, 0x1A, 0xB6, 0x2A, 0xF6, 0x2B, 0x34, 0x2B, 0x74, 0x3B, 0x74, 0x6B, 0xF4,
0x39, 0, 2, 0xC0, 0x11,
0x39, 0, 2, 0x9F, 0x04,
0x39, 0, 3, 0xB2, 0x01, 0x80,
0x39, 0, 19, 0xB3, 0x00, 0x00, 0x10, 0x00, 0x30, 0x50, 0x00, 0x70, 0x90, 0x00, 0xB0, 0xF0, 0x11, 0x30, 0x70, 0x12, 0xF0, 0xF0,
0x39, 0, 21, 0xB4, 0x34, 0xF0, 0xF0, 0x56, 0xF0, 0xF0, 0x78, 0xF0, 0xF0, 0xAB, 0xF0, 0xF0, 0xCD, 0xF0, 0xF0, 0xEF, 0xF0, 0xF0, 0x0F, 0xFF,
0x39, 0, 7, 0xB6, 0x11, 0x00, 0x00, 0x0F, 0xF0, 0x10,
0x39, 0, 3, 0xBA, 0x10, 0x00,
0x39, 0, 5, 0xC4, 0x00, 0x10, 0x00, 0x00,
0x39, 0, 20, 0xC5, 0x00, 0x20, 0x88, 0xC1, 0xB2, 0xB2, 0x32, 0x00, 0x4E, 0x56, 0x35, 0x22, 0x22, 0x2A, 0x05, 0x3F, 0x00, 0x20, 0x0F,
0x39, 0, 13, 0xC6, 0x4B, 0x00, 0x90, 0x00, 0x02, 0xD5, 0x00, 0x02, 0xCB, 0x01, 0x10, 0x43,
0x39, 0, 5, 0xC8, 0x2B, 0x2B, 0x00, 0x00,
0x39, 0, 21, 0xC9, 0x0D, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x1F, 0x1F,
0x39, 0, 21, 0xCA, 0x0D, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x1F, 0x1F,
0x39, 0, 21, 0xCB, 0x0D, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x1F, 0x1F,
0x39, 0, 21, 0xCC, 0x0D, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x1F, 0x1F,
0x39, 0, 21, 0xCD, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C,
0x39, 0, 21, 0xCE, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C,
0x39, 0, 21, 0xCF, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C,
0x39, 0, 21, 0xD0, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C,
0x39, 0, 21, 0xD1, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C,
0x39, 0, 21, 0xD2, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C,
0x39, 0, 21, 0xD3, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C,
0x39, 0, 21, 0xD4, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C,
0x39, 0, 14, 0xB5, 0x11, 0x17, 0x27, 0x1C, 0xd0, 0x00, 0x64, 0x64, 0x00, 0x16, 0xD1, 0x89, 0x0B,
0x39, 0, 10, 0xF8, 0x3E, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA,
0x39, 0, 10, 0xF9, 0xE3, 0x00, 0x00, 0x00, 0xBB, 0xBB, 0xDD, 0x00, 0x00,
0x39, 0, 10, 0xFA, 0x3E, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA,
0x39, 0, 2, 0xB9, 0x90,
0x39, 0, 2, 0x9F, 0x05,
0x39, 0, 7, 0xB2, 0x24, 0x20, 0x05, 0x0F, 0xFF, 0xFF,
0x39, 0, 9, 0xB3, 0xA0, 0x00, 0x28, 0x00, 0x00, 0x00, 0x3F, 0xFF,
0x39, 0, 13, 0xB4, 0x00, 0x60, 0x0E, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x66, 0x8F,
0x39, 0, 21, 0xB5, 0x0F, 0xFF, 0x07, 0xFF, 0x03, 0x33, 0x01, 0x47, 0x01, 0x46, 0x00, 0xC0, 0x00, 0x62, 0x00, 0x31, 0x00, 0x18, 0x00, 0x08,
0x39, 0, 21, 0xB6, 0x0F, 0xFF, 0x07, 0xFF, 0x03, 0x33, 0x00, 0x00, 0x0F, 0xFF, 0x07, 0xFF, 0x03, 0x33, 0x00, 0x00, 0x0F, 0xFF, 0x00, 0x00,
0x39, 0, 16, 0xB9, 0x71, 0x20, 0x05, 0x78, 0x70, 0x5A, 0x3C, 0x78, 0x70, 0x5A, 0x3C, 0x78, 0x70, 0x5A, 0x3C,
0x39, 0, 2, 0x9F, 0x06,
0x39, 0, 41, 0xB2, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 41, 0xB3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 21, 0xB4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x39, 0, 41, 0xB5, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF,
0x39, 0, 41, 0xB6, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF,
0x39, 0, 21, 0xB7, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF,
0x39, 0, 2, 0x9F, 0x04,
0x39, 0, 19, 0xB3, 0x00, 0x00, 0x10, 0x00, 0x30, 0x50, 0x00, 0x70, 0xF0, 0x11, 0x70, 0xF0, 0x23, 0xF0, 0xF0, 0x46, 0xF0, 0xF0,
0x39, 0, 21, 0xB4, 0x8A, 0xF0, 0xF0, 0xCE, 0xF0, 0xF0, 0xFF, 0xF0, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x0F, 0xFF,
0x39, 0, 2, 0x9F, 0x05,
0x39, 0, 21, 0xB5, 0x0F, 0xFF, 0x07, 0xFF, 0x03, 0x33, 0x01, 0x47, 0x01, 0x46, 0x00, 0xCD, 0x00, 0x66, 0x00, 0x3A, 0x00, 0x14, 0x00, 0x08,
0x39, 0, 21, 0xB6, 0x0F, 0xFF, 0x07, 0xFF, 0x03, 0x33, 0x00, 0x00, 0x0F, 0xFE, 0x07, 0x5F, 0x01, 0x64, 0x00, 0x00, 0x0F, 0xFF, 0x00, 0x00,
0x39, 0, 2, 0x9F, 0x05,
0x39, 0, 2, 0xF9, 0x80,
0x39, 0, 4, 0xFF, 0x00, 0x00, 0x01,
0x39, 0, 2, 0x9F, 0x06,
0x39, 0, 3, 0xFF, 0x00, 0x01,
0x39, 0, 5, 0xFE, 0x01, 0x00, 0x00, 0x00,
0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x02, 0x04, 0xBD, 0x33, 0x62, 0xD5, 0x44, 0x7E, 0xFC, 0x56, 0x69, 0x1C, 0x67, 0xAD, 0x30,
0x39, 0, 18, 0xF2, 0x78, 0xA5, 0x17, 0x88, 0x4E, 0x4E, 0x88, 0x4E, 0x4E, 0x88, 0x4E, 0x4E, 0x88, 0x4E, 0x4E, 0x80, 0x4E,
0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x02, 0xD5, 0x9D, 0x33, 0x31, 0x9F, 0x44, 0x3D, 0xB5, 0x55, 0x1D, 0xC3, 0x66, 0x4E, 0xC7,
0x39, 0, 18, 0xF4, 0x77, 0x33, 0x9B, 0x77, 0xCE, 0xCE, 0x77, 0xCE, 0xCE, 0x77, 0xCE, 0xCE, 0x77, 0xCE, 0xCE, 0x70, 0xCE,
0x39, 0, 22, 0xF5, 0x12, 0xAB, 0x0F, 0x22, 0x10, 0x11, 0x23, 0x89, 0x8C, 0x44, 0x25, 0x9D, 0x55, 0x4E, 0xD6, 0x67, 0x4D, 0x0B, 0x78, 0xAC, 0x3D,
0x39, 0, 18, 0xF6, 0x89, 0xC1, 0x3E, 0x99, 0x7C, 0x7C, 0x99, 0x7C, 0x7C, 0x99, 0x7C, 0x7C, 0x99, 0x7C, 0x7C, 0x90, 0x7C,
0x39, 0, 2, 0xFF, 0x01,
0x39, 0, 5, 0xFE, 0x01, 0x01, 0x00, 0x00,
0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x02, 0x04, 0x4B, 0x33, 0x05, 0x76, 0x44, 0x19, 0x92, 0x45, 0xF5, 0x9A, 0x66, 0x20, 0x93,
0x39, 0, 18, 0xF2, 0x67, 0xFB, 0x5C, 0x77, 0x8C, 0x8C, 0x77, 0x8C, 0x8C, 0x77, 0x8C, 0x8C, 0x77, 0x8C, 0x8C, 0x70, 0x8C,
0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x02, 0x04, 0x3E, 0x23, 0xDD, 0x45, 0x34, 0xDF, 0x51, 0x45, 0xAF, 0x49, 0x56, 0xC6, 0x33,
0x39, 0, 18, 0xF4, 0x66, 0x97, 0xF0, 0x77, 0x1B, 0x1B, 0x77, 0x1B, 0x1B, 0x77, 0x1B, 0x1B, 0x77, 0x1B, 0x1B, 0x70, 0x1B,
0x39, 0, 22, 0xF5, 0x11, 0x51, 0xB5, 0x11, 0xB6, 0xB7, 0x23, 0x35, 0x35, 0x34, 0xCD, 0x3C, 0x45, 0xE2, 0x63, 0x56, 0xCF, 0x7F, 0x77, 0x10, 0x90,
0x39, 0, 18, 0xF6, 0x88, 0x04, 0x6D, 0x88, 0xA2, 0xA2, 0x88, 0xA2, 0xA2, 0x88, 0xA2, 0xA2, 0x88, 0xA2, 0xA2, 0x80, 0xA2,
0x39, 0, 2, 0xFF, 0x01,
0x39, 0, 5, 0xFE, 0x01, 0x02, 0x00, 0x00,
0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x01, 0x04, 0x71, 0x23, 0x9F, 0x14, 0x34, 0xB3, 0x27, 0x45, 0x86, 0x1E, 0x56, 0x97, 0x02,
0x39, 0, 18, 0xF2, 0x66, 0x60, 0xB4, 0x66, 0xDF, 0xDF, 0x66, 0xDF, 0xDF, 0x66, 0xDF, 0xDF, 0x66, 0xDF, 0xDF, 0x60, 0xDF,
0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x01, 0x04, 0xC4, 0x22, 0x81, 0xEB, 0x33, 0x7F, 0xEC, 0x44, 0x44, 0xD2, 0x55, 0x47, 0xAA,
0x39, 0, 18, 0xF4, 0x66, 0x04, 0x54, 0x66, 0x7A, 0x7A, 0x66, 0x7A, 0x7A, 0x66, 0x7A, 0x7A, 0x66, 0x7A, 0x7A, 0x60, 0x7A,
0x39, 0, 22, 0xF5, 0x00, 0x20, 0x84, 0x00, 0x85, 0x86, 0x12, 0xCD, 0xE1, 0x33, 0x72, 0xDE, 0x44, 0x7B, 0xF2, 0x55, 0x57, 0xF9, 0x66, 0x7E, 0xEF,
0x39, 0, 18, 0xF6, 0x77, 0x56, 0xB5, 0x77, 0xE2, 0xE2, 0x77, 0xE2, 0xE2, 0x77, 0xE2, 0xE2, 0x77, 0xE2, 0xE2, 0x70, 0xE2,
0x39, 0, 2, 0xFF, 0x01,
0x39, 0, 5, 0xFE, 0x01, 0x03, 0x00, 0x00,
0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x12, 0x8B, 0x66, 0x33, 0x14, 0x85, 0x34, 0xDE, 0x65, 0x45, 0xD2, 0x2D,
0x39, 0, 18, 0xF2, 0x55, 0x7E, 0xC9, 0x55, 0xEA, 0xEA, 0x55, 0xEA, 0xEA, 0x55, 0xEA, 0xEA, 0x55, 0xEA, 0xEA, 0x50, 0xEA,
0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x12, 0xCB, 0x53, 0x23, 0xEB, 0x51, 0x34, 0xA3, 0x24, 0x44, 0x8D, 0xE2,
0x39, 0, 18, 0xF4, 0x55, 0x30, 0x74, 0x55, 0x95, 0x95, 0x55, 0x95, 0x95, 0x55, 0x95, 0x95, 0x55, 0x95, 0x95, 0x50, 0x95,
0x39, 0, 22, 0xF5, 0x00, 0x00, 0x3D, 0x00, 0x3E, 0x3F, 0x02, 0x40, 0x52, 0x23, 0xE3, 0x4A, 0x34, 0xDE, 0x4B, 0x45, 0xA3, 0x33, 0x56, 0xA8, 0x0E,
0x39, 0, 18, 0xF6, 0x66, 0x63, 0xB2, 0x66, 0xD8, 0xD8, 0x66, 0xD8, 0xD8, 0x66, 0xD8, 0xD8, 0x66, 0xD8, 0xD8, 0x60, 0xD8,
0x39, 0, 2, 0xFF, 0x01,
0x39, 0, 5, 0xFE, 0x01, 0x04, 0x00, 0x00,
0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x02, 0x65, 0x47, 0x33, 0x25, 0xA4, 0x44, 0x08, 0x9E, 0x55, 0x13, 0x78,
0x39, 0, 18, 0xF2, 0x56, 0xD2, 0x22, 0x66, 0x48, 0x48, 0x66, 0x48, 0x48, 0x66, 0x48, 0x48, 0x66, 0x48, 0x48, 0x60, 0x48,
0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x02, 0xAC, 0x07, 0x23, 0xE2, 0x64, 0x34, 0xC5, 0x56, 0x45, 0xC7, 0x25,
0x39, 0, 18, 0xF4, 0x55, 0x7A, 0xC7, 0x55, 0xE9, 0xE9, 0x55, 0xE9, 0xE9, 0x55, 0xE9, 0xE9, 0x55, 0xE9, 0xE9, 0x50, 0xE9,
0x39, 0, 22, 0xF5, 0x00, 0x00, 0x3A, 0x00, 0x3B, 0x3C, 0x02, 0x3D, 0x27, 0x23, 0xE3, 0x5F, 0x44, 0x04, 0x7E, 0x45, 0xE0, 0x7C, 0x56, 0xFB, 0x67,
0x39, 0, 18, 0xF6, 0x67, 0xC6, 0x1E, 0x77, 0x44, 0x44, 0x77, 0x44, 0x44, 0x77, 0x44, 0x44, 0x77, 0x44, 0x44, 0x70, 0x44,
0x39, 0, 2, 0xFF, 0x01,	
0x39, 0, 5, 0xFE, 0x01, 0x05, 0x00, 0x00,
0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x23, 0x09, 0x39, 0x34, 0xC6, 0x78, 0x45, 0xFB, 0x64,
0x39, 0, 18, 0xF2, 0x56, 0xC3, 0x16, 0x66, 0x3E, 0x3E, 0x66, 0x3E, 0x3E, 0x66, 0x3E, 0x3E, 0x66, 0x3E, 0x3E, 0x60, 0x3E,
0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x23, 0x6E, 0x25, 0x34, 0x9D, 0x3F, 0x45, 0xB6, 0x1C,
0x39, 0, 18, 0xF4, 0x55, 0x72, 0xC1, 0x55, 0xE5, 0xE5, 0x55, 0xE5, 0xE5, 0x55, 0xE5, 0xE5, 0x55, 0xE5, 0xE5, 0x50, 0xE5,
0x39, 0, 22, 0xF5, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x12, 0xE5, 0xF9, 0x34, 0xD7, 0x61, 0x45, 0xCB, 0x6F, 0x56, 0xF2, 0x62,
0x39, 0, 18, 0xF6, 0x67, 0xC2, 0x18, 0x77, 0x41, 0x41, 0x77, 0x41, 0x41, 0x77, 0x41, 0x41, 0x77, 0x41, 0x41, 0x70, 0x41,
0x39, 0, 2, 0xFF, 0x01,
0x39, 0, 5, 0xFE, 0x01, 0x06, 0x00, 0x00,
0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x01, 0x08, 0xE6, 0x34, 0x26, 0x18, 0x45, 0xAA, 0x1B,
0x39, 0, 18, 0xF2, 0x55, 0x7C, 0xD2, 0x55, 0xFC, 0xFC, 0x55, 0xFC, 0xFC, 0x55, 0xFC, 0xFC, 0x55, 0xFC, 0xFC, 0x50, 0xFC,
0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x02, 0x3D, 0x75, 0x33, 0x21, 0xE7, 0x44, 0x6C, 0xD4,
0x39, 0, 18, 0xF4, 0x55, 0x30, 0x81, 0x55, 0xA6, 0xA6, 0x55, 0xA6, 0xA6, 0x55, 0xA6, 0xA6, 0x55, 0xA6, 0xA6, 0x50, 0xA6,
0x39, 0, 22, 0xF5, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x11, 0xC9, 0xCA, 0x34, 0x64, 0x0F, 0x45, 0x82, 0x30, 0x56, 0xB9, 0x29,
0x39, 0, 18, 0xF6, 0x66, 0x8A, 0xDD, 0x77, 0x06, 0x06, 0x77, 0x06, 0x06, 0x77, 0x06, 0x06, 0x77, 0x06, 0x06, 0x70, 0x06,
0x39, 0, 2, 0xFF, 0x01,
0x39, 0, 5, 0xFE, 0x01, 0x07, 0x00, 0x00,
0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x13, 0x7B, 0x9B, 0x44, 0x56, 0xD3,
0x39, 0, 18, 0xF2, 0x55, 0x3E, 0x9B, 0x55, 0xC5, 0xC5, 0x55, 0xC5, 0xC5, 0x55, 0xC5, 0xC5, 0x55, 0xC5, 0xC5, 0x50, 0xC5,
0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0xDC, 0x23, 0x6F, 0x83, 0x44, 0x21, 0x93,
0x39, 0, 18, 0xF4, 0x45, 0xF4, 0x4E, 0x55, 0x73, 0x73, 0x55, 0x73, 0x73, 0x55, 0x73, 0x73, 0x55, 0x73, 0x73, 0x50, 0x73,
0x39, 0, 22, 0xF5, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x33, 0x17, 0xAE, 0x44, 0x3A, 0xF9, 0x55, 0x8B, 0xFF,
0x39, 0, 18, 0xF6, 0x66, 0x62, 0xB8, 0x66, 0xE0, 0xE0, 0x66, 0xE0, 0xE0, 0x66, 0xE0, 0xE0, 0x66, 0xE0, 0xE0, 0x60, 0xE0,
0x39, 0, 2, 0xFF, 0x01,
0x39, 0, 5, 0xFE, 0x01, 0x08, 0x00, 0x00,
0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x00, 0x0A, 0x0B, 0x23, 0xB5, 0xAF,
0x39, 0, 18, 0xF2, 0x44, 0x38, 0xA1, 0x44, 0xCD, 0xCD, 0x44, 0xCD, 0xCD, 0x44, 0xCD, 0xCD, 0x44, 0xCD, 0xCD, 0x40, 0xCD,
0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x01, 0x0A, 0xC0, 0x23, 0xF3, 0x98,
0x39, 0, 18, 0xF4, 0x44, 0x0C, 0x65, 0x44, 0x8F, 0x8F, 0x44, 0x8F, 0x8F, 0x44, 0x8F, 0x8F, 0x44, 0x8F, 0x8F, 0x40, 0x8F,
0x39, 0, 22, 0xF5, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x01, 0x08, 0x82, 0x34, 0x2E, 0x12, 0x45, 0xB0, 0x2C,
0x39, 0, 18, 0xF6, 0x55, 0x93, 0xEC, 0x66, 0x13, 0x13, 0x66, 0x13, 0x13, 0x66, 0x13, 0x13, 0x66, 0x13, 0x13, 0x60, 0x13,
0x39, 0, 2, 0xFF, 0x01,
0x39, 0, 5, 0xFE, 0x01, 0x09, 0x00, 0x00,
0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x00, 0x0A, 0x0B, 0x00, 0x0C, 0x0D,
0x39, 0, 18, 0xF2, 0x23, 0x73, 0x5A, 0x33, 0xA3, 0xA3, 0x33, 0xA3, 0xA3, 0x33, 0xA3, 0xA3, 0x33, 0xA3, 0xA3, 0x30, 0xA3,
0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x00, 0x0A, 0x0B, 0x02, 0x0C, 0x0A,
0x39, 0, 18, 0xF4, 0x23, 0xDC, 0x60, 0x33, 0x94, 0x94, 0x33, 0x94, 0x94, 0x33, 0x94, 0x94, 0x33, 0x94, 0x94, 0x30, 0x94,
0x39, 0, 22, 0xF5, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x03, 0x0A, 0x03, 0x34, 0xBB, 0x51,
0x39, 0, 18, 0xF6, 0x45, 0xBE, 0x1D, 0x55, 0x47, 0x47, 0x55, 0x47, 0x47, 0x55, 0x47, 0x47, 0x55, 0x47, 0x47, 0x50, 0x47,
0x39, 0, 2, 0xFF, 0x01,
0x39, 0, 2, 0x9F, 0x05,
0x39, 0, 2, 0xF9, 0x00,
0x39, 0, 4, 0xFF, 0x00, 0x00, 0x00,
0x39, 0, 2, 0x9F, 0x06,
0x39, 0, 3, 0xFF, 0x00, 0x00,
0x39, 0, 2, 0x9F, 0x07,
0x39, 0, 8, 0xB2, 0x10, 0x20, 0x20, 0x20, 0x02, 0x00, 0x99,
0x39, 0, 8, 0xB3, 0x10, 0x10, 0x10, 0x10, 0x02, 0x00, 0x99,
0x39, 0, 8, 0xB4, 0x10, 0x10, 0x10, 0x10, 0x02, 0x00, 0x99,
0x39, 0, 8, 0xB5, 0x10, 0x10, 0x10, 0x10, 0x02, 0x00, 0x99,
0x39, 0, 8, 0xB6, 0x04, 0x04, 0x04, 0x04, 0x02, 0x02, 0x64,
0x39, 0, 8, 0xB7, 0x10, 0x10, 0x10, 0x10, 0x02, 0x00, 0x99,
0x39, 0, 8, 0xB8, 0x10, 0x10, 0x10, 0x10, 0x02, 0x00, 0x99,
0x39, 0, 17, 0xB9, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10,
0x39, 0, 17, 0xBA, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10,
0x39, 0, 17, 0xBB, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10,
0x39, 0, 17, 0xBC, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10,
0x39, 0, 17, 0xBD, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10,
0x39, 0, 17, 0xBE, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10,
0x39, 0, 17, 0xBF, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10,
0x39, 0, 16, 0xC0, 0x00, 0x99, 0x99, 0x00, 0x99, 0x99, 0x00, 0x64, 0x3F, 0x00, 0x25, 0x17, 0x00, 0x10, 0x0C,
0x39, 0, 16, 0xC1, 0x00, 0x99, 0x99, 0x00, 0x99, 0x99, 0x00, 0x64, 0x3F, 0x00, 0x25, 0x17, 0x00, 0x10, 0x0C,
0x39, 0, 16, 0xC2, 0x00, 0x99, 0x99, 0x00, 0x99, 0x99, 0x00, 0x64, 0x3F, 0x00, 0x25, 0x17, 0x00, 0x10, 0x0C,
0x39, 0, 16, 0xC3, 0x00, 0x99, 0x99, 0x00, 0x99, 0x99, 0x00, 0x64, 0x3F, 0x00, 0x25, 0x17, 0x00, 0x10, 0x0C,
0x39, 0, 7, 0xC4, 0x22, 0x58, 0x58, 0x22, 0x58, 0x58,
0x39, 0, 7, 0xC5, 0x00, 0x99, 0x99, 0x00, 0x99, 0x99,
0x39, 0, 4, 0xC6, 0x00, 0xC8, 0xC8,
0x39, 0, 2, 0x9F, 0x0F,
0x39, 0, 2, 0xF8, 0x00,
0x39, 0, 2, 0x9F, 0x0A,
0x39, 0, 8, 0xB2, 0x1F, 0x00, 0x00, 0x20, 0x01, 0x10, 0x00,
0x39, 0, 46, 0xB8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00,
0x39, 0, 43, 0xB9, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00,
0x39, 0, 43, 0xBA, 0x10, 0x10, 0x00, 0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x00, 0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00,
0x39, 0, 2, 0x9F, 0x0B,
0x15, 0, 9, 0xB2, 0x01, 0x2F, 0x4F, 0x00, 0x00, 0x00, 0x00, 0x00,
0x15, 0, 54, 0xB9, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x70, 0x24, 0x0A, 0x6A, 0x13, 0x9A, 0xFA, 0x48, 0x14, 0xDA, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x15, 0, 54, 0xBA, 0x22, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x20, 0x20, 0x0A, 0x32, 0x00, 0x00, 0x00, 0x48, 0x38, 0xE8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x15, 0, 20, 0xC8, 0x0F, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x08, 0x08, 0x08, 0x08, 0x00, 0x80, 0x80, 0x80, 0x80,	
0x39, 0, 2, 0x48, 0x03,         
0x39, 0, 3, 0x51, 0x00, 0x00,
0x39, 0, 2, 0x53, 0xE0,
0x39, 0, 1, 0x35,
//0x39, 0, 3, 0x51, 0x03, 0x40
};

/**
* @brief  panel init
* @param  none
* @retval none
*/
static void app_init_panel(void)
{
    /* reset panel*/
    app_tx_panel_reset();
//	 send_panel_init_code( sizeof( panel_init_code ), panel_init_code );

#if 1
//2023-12-11
    //#######Initial Code Start#########
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0x9C, 0xA5, 0xA5);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0xFD, 0x5A, 0x5A);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x0F);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xB3, 0x00);	//OTP Reload Enable
	//##========================Group 1 Timing=============================
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x01);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 4, 0xB2, 0x51, 0x04, 0x69);	//1044*2260 6.0
	////NOR:60Hz
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 36, 0xB3, 0x01, 0x6F, 0x00, 0x0A, 0x2E, 0x00, 0x01, 0x6F, 0x00, 0x0A, 0x2E, 0x00, 0x01, 0x6F, 0x00, 0x0A, 0x2E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x6F, 0x00, 0x0A, 0x14, 0x44, 0x1C, 0x1C, 0x1C, 0x18, 0x1C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0xD9, 0x01, 0x1F);	//For demura MEM and MIPI
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 4, 0xCF, 0x01, 0x00, 0x00);	//AOD MEM no reset
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xD0, 0x01);		//TMC MODE
//    hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xD3, 0x01);	// command mode 0x01
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xD3, 0x00);		// video mode 需要关闭 encoder 3x disable
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xD5, 0x03);		// video mode RAM bypass
	//##========================Group 2 PWR&GOA=============================
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x02);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 26, 0xB2, 0x00, 0xC1, 0x32, 0x10, 0x11, 0x12, 0x12, 0x00, 0x08, 0x78, 0x21, 0x11, 0x14, 0x13, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x21, 0x21, 0x11, 0x1A);	//power sequence
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xB3, 0x0B);   //OSC128 & OSC155 Enable
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 14, 0xB4, 0x00, 0x00, 0x8F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00); //VDD_EXT_CONT=1,VDD_VEXT_SEL=00(1.15V)0A(1.1)0F(1.05) VDDL=1.15V  
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 7, 0xB5, 0x00, 0x2F, 0x2F, 0x2F, 0x2F, 0x2F);    //VGH=7.7V setting  20220825
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 20, 0xB6, 0x01, 0x00, 0x00, 0x11, 0x11, 0x11, 0x1C, 0x08, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21);
//	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xB7, 0x00, 0x10, 0x00, 0x12, 0x10, 0x15, 0x00, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xB7, 0x00, 0x10, 0x15, 0x12, 0x10, 0x15, 0x00, 0x06, 0x03, 0x03, 0x06, 0x03, 0x03, 0x06, 0x03, 0x03);
	
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 14, 0xB8, 0x00, 0x00, 0x00, 0x73, 0x2D, 0x73, 0x2D, 0x73, 0x2D, 0x73, 0x2D, 0x73, 0x2D);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 9, 0xB9, 0x00, 0x00, 0x50, 0x00, 0x00, 0x17, 0x17, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 9, 0xBA, 0x08, 0xFF, 0x36, 0x03, 0x36, 0x03, 0x11, 0x01);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 26, 0xBB, 0x00, 0xCD, 0x0F, 0xC3, 0x64, 0x00, 0xCD, 0x0F, 0xC3, 0x64, 0x00, 0xCD, 0x0F, 0xC3, 0x64, 0x00, 0xCD, 0x0F, 0xC3, 0x64, 0x00, 0xCD, 0x64, 0x6E, 0x64);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 6, 0xBE, 0x5F, 0x16, 0x04, 0x5F, 0x75);  //IP Gating ON/Off
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 6, 0xBF, 0x08, 0x00, 0x10, 0x10, 0x08);  //TE 60 Porch
    //##========================GOA & SOURCE Fixed by Pinchao=============================
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xC1, 0x23, 0x00, 0x24, 0x00, 0x05, 0x00, 0x3D, 0x21, 0x00, 0x3D, 0x00, 0x01, 0x00, 0x3D, 0x00, 0x22);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xC2, 0x23, 0x00, 0x24, 0x00, 0x05, 0x00, 0x3D, 0x21, 0x00, 0x3D, 0x00, 0x01, 0x00, 0x3D, 0x00, 0x22);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 8, 0xC3, 0x11, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xC4, 0x00, 0x05, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x1f, 0x00, 0x00, 0x08);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 27, 0xC5, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 27, 0xC6, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x05, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0x55, 0x00, 0x01, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 14, 0xC7, 0x00, 0x1A, 0x25, 0x24, 0x20, 0x00, 0x24, 0x20, 0x24, 0x20, 0x00, 0x20, 0x20);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 14, 0xC8, 0x00, 0x35, 0x1A, 0x20, 0x20, 0x00, 0x20, 0x20, 0x20, 0x20, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 27, 0xC9, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 27, 0xCA, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 27, 0xCB, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	//hal_dsi_tx_ctrl_write_cmd(0x39, 0, 27, 0xCC, 0x04, 0x1A, 0x1B, 0x1A, 0xCD, 0x00, 0x1A, 0xCD, 0x1A, 0xCD, 0x00, 0x24, 0x9F, 0x00, 0x19, 0xBA, 0x20, 0x68, 0x00, 0x24, 0x68, 0x00, 0x00, 0x00, 0x24, 0x6B);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 27, 0xCC, 0x04, 0x3A, 0x3B, 0x1A, 0xCD, 0x00, 0x1A, 0xCD, 0x1A, 0xCD, 0x00, 0x44, 0xBF, 0x00, 0x19, 0xBA, 0x20, 0x68, 0x00, 0x24, 0x68, 0x00, 0x00, 0x00, 0x24, 0x6B);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 25, 0xCD, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 25, 0xCE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 25, 0xCF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 25, 0xD0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 25, 0xD1, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
//    hal_dsi_tx_ctrl_write_cmd(0x39, 0, 25, 0xD1, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xD2, 0x00, 0x00, 0x0F, 0x00, 0x00, 0x09, 0x00, 0x00, 0x09, 0x00, 0x00, 0x09, 0x00, 0x00, 0x09, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xD3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 15, 0xD7, 0x44, 0x01, 0x3F, 0x24, 0x56, 0x24, 0x56, 0x01, 0x11, 0x99, 0x0A, 0x00, 0x00, 0x03);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 26, 0xD8, 0x00, 0x00, 0xC5, 0x9F, 0x40, 0x00, 0x00, 0xC5, 0x9F, 0x40, 0x00, 0x00, 0xC5, 0x9F, 0x40, 0x00, 0x00, 0xC5, 0x9F, 0x40, 0x00, 0x00, 0xC5, 0x00, 0x00);
//	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 12, 0xD9, 0x10, 0x00, 0x33, 0x00, 0x33, 0x00, 0x33, 0x00, 0x33, 0x00, 0x30);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 12, 0xD9, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 6, 0xE1, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 26, 0xE2, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 13, 0xE3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 9, 0xE7, 0x80, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x22);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 12, 0xEE, 0x00, 0x00, 0x00, 0xBB, 0x58, 0x08, 0xA2, 0x50, 0x88, 0x88, 0x29);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 6, 0xEF, 0x00, 0x80, 0x82, 0x80, 0x85);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x03);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 33, 0xB2, 0x11, 0x00, 0x00, 0x89, 0x30, 0x80, 0x09, 0x60, 0x04, 0x38, 0x00, 0x0C, 0x04, 0x38, 0x04, 0x38, 0x02, 0x00, 0x03, 0x1C, 0x00, 0x20, 0x01, 0x7E, 0x00, 0x0F, 0x00, 0x0C, 0x08, 0xBB, 0x04, 0x3D);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 33, 0xB3, 0x18, 0x00, 0x10, 0xF0, 0x03, 0x0C, 0x20, 0x00, 0x06, 0x0B, 0x0B, 0x33, 0x0E, 0x1C, 0x2A, 0x38, 0x46, 0x54, 0x62, 0x69, 0x70, 0x77, 0x79, 0x7B, 0x7D, 0x7E, 0x01, 0x02, 0x01, 0x00, 0x09, 0x40);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 25, 0xB4, 0x09, 0xBE, 0x19, 0xFC, 0x19, 0xFA, 0x19, 0xF8, 0x1A, 0x38, 0x1A, 0x78, 0x1A, 0xB6, 0x2A, 0xF6, 0x2B, 0x34, 0x2B, 0x74, 0x3B, 0x74, 0x6B, 0xF4);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xC0, 0x11);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x04);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0xB2, 0x01, 0x80);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 19, 0xB3, 0x00, 0x00, 0x10, 0x00, 0x30, 0x50, 0x00, 0x70, 0x90, 0x00, 0xB0, 0xF0, 0x11, 0x30, 0x70, 0x12, 0xF0, 0xF0);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xB4, 0x34, 0xF0, 0xF0, 0x56, 0xF0, 0xF0, 0x78, 0xF0, 0xF0, 0xAB, 0xF0, 0xF0, 0xCD, 0xF0, 0xF0, 0xEF, 0xF0, 0xF0, 0x0F, 0xFF);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 7, 0xB6, 0x11, 0x00, 0x00, 0x0F, 0xF0, 0x10);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0xBA, 0x10, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xC4, 0x00, 0x10, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 20, 0xC5, 0x00, 0x20, 0x88, 0xC1, 0xB2, 0xB2, 0x32, 0x00, 0x4E, 0x56, 0x35, 0x22, 0x22, 0x2A, 0x05, 0x3F, 0x00, 0x20, 0x0F);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 13, 0xC6, 0x4B, 0x00, 0x90, 0x00, 0x02, 0xD5, 0x00, 0x02, 0xCB, 0x01, 0x10, 0x43);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xC8, 0x2B, 0x2B, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xC9, 0x0D, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x1F, 0x1F);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xCA, 0x0D, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x1F, 0x1F);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xCB, 0x0D, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x1F, 0x1F);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xCC, 0x0D, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x23, 0x1F, 0x1F);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xCD, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xCE, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xCF, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xD0, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xD1, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xD2, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xD3, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xD4, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x21, 0x1C, 0x1C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 14, 0xB5, 0x11, 0x17, 0x27, 0x1C, 0xd0, 0x00, 0x64, 0x64, 0x00, 0x16, 0xD1, 0x89, 0x0B);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 10, 0xF8, 0x3E, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 10, 0xF9, 0xE3, 0x00, 0x00, 0x00, 0xBB, 0xBB, 0xDD, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 10, 0xFA, 0x3E, 0xAA, 0xAA, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xB9, 0x90);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x05);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 7, 0xB2, 0x24, 0x20, 0x05, 0x0F, 0xFF, 0xFF);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 9, 0xB3, 0xA0, 0x00, 0x28, 0x00, 0x00, 0x00, 0x3F, 0xFF);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 13, 0xB4, 0x00, 0x60, 0x0E, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x66, 0x8F);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xB5, 0x0F, 0xFF, 0x07, 0xFF, 0x03, 0x33, 0x01, 0x47, 0x01, 0x46, 0x00, 0xC0, 0x00, 0x62, 0x00, 0x31, 0x00, 0x18, 0x00, 0x08);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xB6, 0x0F, 0xFF, 0x07, 0xFF, 0x03, 0x33, 0x00, 0x00, 0x0F, 0xFF, 0x07, 0xFF, 0x03, 0x33, 0x00, 0x00, 0x0F, 0xFF, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 16, 0xB9, 0x71, 0x20, 0x05, 0x78, 0x70, 0x5A, 0x3C, 0x78, 0x70, 0x5A, 0x3C, 0x78, 0x70, 0x5A, 0x3C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x06);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 41, 0xB2, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 41, 0xB3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xB4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 41, 0xB5, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 41, 0xB6, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xB7, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF, 0x01, 0xFF);

	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x04);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 19, 0xB3, 0x00, 0x00, 0x10, 0x00, 0x30, 0x50, 0x00, 0x70, 0xF0, 0x11, 0x70, 0xF0, 0x23, 0xF0, 0xF0, 0x46, 0xF0, 0xF0);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xB4, 0x8A, 0xF0, 0xF0, 0xCE, 0xF0, 0xF0, 0xFF, 0xF0, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x0F, 0xFF);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x05);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xB5, 0x0F, 0xFF, 0x07, 0xFF, 0x03, 0x33, 0x01, 0x47, 0x01, 0x46, 0x00, 0xCD, 0x00, 0x66, 0x00, 0x3A, 0x00, 0x14, 0x00, 0x08);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 21, 0xB6, 0x0F, 0xFF, 0x07, 0xFF, 0x03, 0x33, 0x00, 0x00, 0x0F, 0xFE, 0x07, 0x5F, 0x01, 0x64, 0x00, 0x00, 0x0F, 0xFF, 0x00, 0x00);

	//--------------------------------------------------------------
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x05);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xF9, 0x80);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 4, 0xFF, 0x00, 0x00, 0x01);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x06);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0xFF, 0x00, 0x01);
	//--------------------------------------------------------------
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xFE, 0x01, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x02, 0x04, 0xBD, 0x33, 0x62, 0xD5, 0x44, 0x7E, 0xFC, 0x56, 0x69, 0x1C, 0x67, 0xAD, 0x30);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF2, 0x78, 0xA5, 0x17, 0x88, 0x4E, 0x4E, 0x88, 0x4E, 0x4E, 0x88, 0x4E, 0x4E, 0x88, 0x4E, 0x4E, 0x80, 0x4E);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x02, 0xD5, 0x9D, 0x33, 0x31, 0x9F, 0x44, 0x3D, 0xB5, 0x55, 0x1D, 0xC3, 0x66, 0x4E, 0xC7);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF4, 0x77, 0x33, 0x9B, 0x77, 0xCE, 0xCE, 0x77, 0xCE, 0xCE, 0x77, 0xCE, 0xCE, 0x77, 0xCE, 0xCE, 0x70, 0xCE);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF5, 0x12, 0xAB, 0x0F, 0x22, 0x10, 0x11, 0x23, 0x89, 0x8C, 0x44, 0x25, 0x9D, 0x55, 0x4E, 0xD6, 0x67, 0x4D, 0x0B, 0x78, 0xAC, 0x3D);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF6, 0x89, 0xC1, 0x3E, 0x99, 0x7C, 0x7C, 0x99, 0x7C, 0x7C, 0x99, 0x7C, 0x7C, 0x99, 0x7C, 0x7C, 0x90, 0x7C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xFF, 0x01);
	delayMs(1);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xFE, 0x01, 0x01, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x02, 0x04, 0x4B, 0x33, 0x05, 0x76, 0x44, 0x19, 0x92, 0x45, 0xF5, 0x9A, 0x66, 0x20, 0x93);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF2, 0x67, 0xFB, 0x5C, 0x77, 0x8C, 0x8C, 0x77, 0x8C, 0x8C, 0x77, 0x8C, 0x8C, 0x77, 0x8C, 0x8C, 0x70, 0x8C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x02, 0x04, 0x3E, 0x23, 0xDD, 0x45, 0x34, 0xDF, 0x51, 0x45, 0xAF, 0x49, 0x56, 0xC6, 0x33);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF4, 0x66, 0x97, 0xF0, 0x77, 0x1B, 0x1B, 0x77, 0x1B, 0x1B, 0x77, 0x1B, 0x1B, 0x77, 0x1B, 0x1B, 0x70, 0x1B);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF5, 0x11, 0x51, 0xB5, 0x11, 0xB6, 0xB7, 0x23, 0x35, 0x35, 0x34, 0xCD, 0x3C, 0x45, 0xE2, 0x63, 0x56, 0xCF, 0x7F, 0x77, 0x10, 0x90);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF6, 0x88, 0x04, 0x6D, 0x88, 0xA2, 0xA2, 0x88, 0xA2, 0xA2, 0x88, 0xA2, 0xA2, 0x88, 0xA2, 0xA2, 0x80, 0xA2);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xFF, 0x01);
	delayMs(1);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xFE, 0x01, 0x02, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x01, 0x04, 0x71, 0x23, 0x9F, 0x14, 0x34, 0xB3, 0x27, 0x45, 0x86, 0x1E, 0x56, 0x97, 0x02);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF2, 0x66, 0x60, 0xB4, 0x66, 0xDF, 0xDF, 0x66, 0xDF, 0xDF, 0x66, 0xDF, 0xDF, 0x66, 0xDF, 0xDF, 0x60, 0xDF);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x01, 0x04, 0xC4, 0x22, 0x81, 0xEB, 0x33, 0x7F, 0xEC, 0x44, 0x44, 0xD2, 0x55, 0x47, 0xAA);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF4, 0x66, 0x04, 0x54, 0x66, 0x7A, 0x7A, 0x66, 0x7A, 0x7A, 0x66, 0x7A, 0x7A, 0x66, 0x7A, 0x7A, 0x60, 0x7A);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF5, 0x00, 0x20, 0x84, 0x00, 0x85, 0x86, 0x12, 0xCD, 0xE1, 0x33, 0x72, 0xDE, 0x44, 0x7B, 0xF2, 0x55, 0x57, 0xF9, 0x66, 0x7E, 0xEF);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF6, 0x77, 0x56, 0xB5, 0x77, 0xE2, 0xE2, 0x77, 0xE2, 0xE2, 0x77, 0xE2, 0xE2, 0x77, 0xE2, 0xE2, 0x70, 0xE2);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xFF, 0x01);
	delayMs(1);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xFE, 0x01, 0x03, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x12, 0x8B, 0x66, 0x33, 0x14, 0x85, 0x34, 0xDE, 0x65, 0x45, 0xD2, 0x2D);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF2, 0x55, 0x7E, 0xC9, 0x55, 0xEA, 0xEA, 0x55, 0xEA, 0xEA, 0x55, 0xEA, 0xEA, 0x55, 0xEA, 0xEA, 0x50, 0xEA);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x12, 0xCB, 0x53, 0x23, 0xEB, 0x51, 0x34, 0xA3, 0x24, 0x44, 0x8D, 0xE2);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF4, 0x55, 0x30, 0x74, 0x55, 0x95, 0x95, 0x55, 0x95, 0x95, 0x55, 0x95, 0x95, 0x55, 0x95, 0x95, 0x50, 0x95);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF5, 0x00, 0x00, 0x3D, 0x00, 0x3E, 0x3F, 0x02, 0x40, 0x52, 0x23, 0xE3, 0x4A, 0x34, 0xDE, 0x4B, 0x45, 0xA3, 0x33, 0x56, 0xA8, 0x0E);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF6, 0x66, 0x63, 0xB2, 0x66, 0xD8, 0xD8, 0x66, 0xD8, 0xD8, 0x66, 0xD8, 0xD8, 0x66, 0xD8, 0xD8, 0x60, 0xD8);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xFF, 0x01);
	delayMs(1);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xFE, 0x01, 0x04, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x02, 0x65, 0x47, 0x33, 0x25, 0xA4, 0x44, 0x08, 0x9E, 0x55, 0x13, 0x78);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF2, 0x56, 0xD2, 0x22, 0x66, 0x48, 0x48, 0x66, 0x48, 0x48, 0x66, 0x48, 0x48, 0x66, 0x48, 0x48, 0x60, 0x48);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x02, 0xAC, 0x07, 0x23, 0xE2, 0x64, 0x34, 0xC5, 0x56, 0x45, 0xC7, 0x25);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF4, 0x55, 0x7A, 0xC7, 0x55, 0xE9, 0xE9, 0x55, 0xE9, 0xE9, 0x55, 0xE9, 0xE9, 0x55, 0xE9, 0xE9, 0x50, 0xE9);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF5, 0x00, 0x00, 0x3A, 0x00, 0x3B, 0x3C, 0x02, 0x3D, 0x27, 0x23, 0xE3, 0x5F, 0x44, 0x04, 0x7E, 0x45, 0xE0, 0x7C, 0x56, 0xFB, 0x67);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF6, 0x67, 0xC6, 0x1E, 0x77, 0x44, 0x44, 0x77, 0x44, 0x44, 0x77, 0x44, 0x44, 0x77, 0x44, 0x44, 0x70, 0x44);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xFF, 0x01);
	delayMs(1);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xFE, 0x01, 0x05, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x23, 0x09, 0x39, 0x34, 0xC6, 0x78, 0x45, 0xFB, 0x64);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF2, 0x56, 0xC3, 0x16, 0x66, 0x3E, 0x3E, 0x66, 0x3E, 0x3E, 0x66, 0x3E, 0x3E, 0x66, 0x3E, 0x3E, 0x60, 0x3E);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x23, 0x6E, 0x25, 0x34, 0x9D, 0x3F, 0x45, 0xB6, 0x1C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF4, 0x55, 0x72, 0xC1, 0x55, 0xE5, 0xE5, 0x55, 0xE5, 0xE5, 0x55, 0xE5, 0xE5, 0x55, 0xE5, 0xE5, 0x50, 0xE5);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF5, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x12, 0xE5, 0xF9, 0x34, 0xD7, 0x61, 0x45, 0xCB, 0x6F, 0x56, 0xF2, 0x62);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF6, 0x67, 0xC2, 0x18, 0x77, 0x41, 0x41, 0x77, 0x41, 0x41, 0x77, 0x41, 0x41, 0x77, 0x41, 0x41, 0x70, 0x41);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xFF, 0x01);
	delayMs(1);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xFE, 0x01, 0x06, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x01, 0x08, 0xE6, 0x34, 0x26, 0x18, 0x45, 0xAA, 0x1B);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF2, 0x55, 0x7C, 0xD2, 0x55, 0xFC, 0xFC, 0x55, 0xFC, 0xFC, 0x55, 0xFC, 0xFC, 0x55, 0xFC, 0xFC, 0x50, 0xFC);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x02, 0x3D, 0x75, 0x33, 0x21, 0xE7, 0x44, 0x6C, 0xD4);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF4, 0x55, 0x30, 0x81, 0x55, 0xA6, 0xA6, 0x55, 0xA6, 0xA6, 0x55, 0xA6, 0xA6, 0x55, 0xA6, 0xA6, 0x50, 0xA6);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF5, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x11, 0xC9, 0xCA, 0x34, 0x64, 0x0F, 0x45, 0x82, 0x30, 0x56, 0xB9, 0x29);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF6, 0x66, 0x8A, 0xDD, 0x77, 0x06, 0x06, 0x77, 0x06, 0x06, 0x77, 0x06, 0x06, 0x77, 0x06, 0x06, 0x70, 0x06);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xFF, 0x01);
	delayMs(1);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xFE, 0x01, 0x07, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x13, 0x7B, 0x9B, 0x44, 0x56, 0xD3);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF2, 0x55, 0x3E, 0x9B, 0x55, 0xC5, 0xC5, 0x55, 0xC5, 0xC5, 0x55, 0xC5, 0xC5, 0x55, 0xC5, 0xC5, 0x50, 0xC5);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0xDC, 0x23, 0x6F, 0x83, 0x44, 0x21, 0x93);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF4, 0x45, 0xF4, 0x4E, 0x55, 0x73, 0x73, 0x55, 0x73, 0x73, 0x55, 0x73, 0x73, 0x55, 0x73, 0x73, 0x50, 0x73);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF5, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x33, 0x17, 0xAE, 0x44, 0x3A, 0xF9, 0x55, 0x8B, 0xFF);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF6, 0x66, 0x62, 0xB8, 0x66, 0xE0, 0xE0, 0x66, 0xE0, 0xE0, 0x66, 0xE0, 0xE0, 0x66, 0xE0, 0xE0, 0x60, 0xE0);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xFF, 0x01);
	delayMs(1);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xFE, 0x01, 0x08, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x00, 0x0A, 0x0B, 0x23, 0xB5, 0xAF);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF2, 0x44, 0x38, 0xA1, 0x44, 0xCD, 0xCD, 0x44, 0xCD, 0xCD, 0x44, 0xCD, 0xCD, 0x44, 0xCD, 0xCD, 0x40, 0xCD);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x01, 0x0A, 0xC0, 0x23, 0xF3, 0x98);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF4, 0x44, 0x0C, 0x65, 0x44, 0x8F, 0x8F, 0x44, 0x8F, 0x8F, 0x44, 0x8F, 0x8F, 0x44, 0x8F, 0x8F, 0x40, 0x8F);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF5, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x01, 0x08, 0x82, 0x34, 0x2E, 0x12, 0x45, 0xB0, 0x2C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF6, 0x55, 0x93, 0xEC, 0x66, 0x13, 0x13, 0x66, 0x13, 0x13, 0x66, 0x13, 0x13, 0x66, 0x13, 0x13, 0x60, 0x13);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xFF, 0x01);
	delayMs(1);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 5, 0xFE, 0x01, 0x09, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF1, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x00, 0x0A, 0x0B, 0x00, 0x0C, 0x0D);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF2, 0x23, 0x73, 0x5A, 0x33, 0xA3, 0xA3, 0x33, 0xA3, 0xA3, 0x33, 0xA3, 0xA3, 0x33, 0xA3, 0xA3, 0x30, 0xA3);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF3, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x00, 0x0A, 0x0B, 0x02, 0x0C, 0x0A);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF4, 0x23, 0xDC, 0x60, 0x33, 0x94, 0x94, 0x33, 0x94, 0x94, 0x33, 0x94, 0x94, 0x33, 0x94, 0x94, 0x30, 0x94);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 22, 0xF5, 0x00, 0x00, 0x01, 0x00, 0x02, 0x03, 0x00, 0x04, 0x05, 0x00, 0x06, 0x07, 0x00, 0x08, 0x09, 0x03, 0x0A, 0x03, 0x34, 0xBB, 0x51);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 18, 0xF6, 0x45, 0xBE, 0x1D, 0x55, 0x47, 0x47, 0x55, 0x47, 0x47, 0x55, 0x47, 0x47, 0x55, 0x47, 0x47, 0x50, 0x47);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xFF, 0x01);
	delayMs(1);

	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x05);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xF9, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 4, 0xFF, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x06);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0xFF, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x07);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 8, 0xB2, 0x10, 0x20, 0x20, 0x20, 0x02, 0x00, 0x99);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 8, 0xB3, 0x10, 0x10, 0x10, 0x10, 0x02, 0x00, 0x99);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 8, 0xB4, 0x10, 0x10, 0x10, 0x10, 0x02, 0x00, 0x99);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 8, 0xB5, 0x10, 0x10, 0x10, 0x10, 0x02, 0x00, 0x99);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 8, 0xB6, 0x04, 0x04, 0x04, 0x04, 0x02, 0x02, 0x64);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 8, 0xB7, 0x10, 0x10, 0x10, 0x10, 0x02, 0x00, 0x99);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 8, 0xB8, 0x10, 0x10, 0x10, 0x10, 0x02, 0x00, 0x99);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xB9, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xBA, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xBB, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xBC, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xBD, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xBE, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 17, 0xBF, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 16, 0xC0, 0x00, 0x99, 0x99, 0x00, 0x99, 0x99, 0x00, 0x64, 0x3F, 0x00, 0x25, 0x17, 0x00, 0x10, 0x0C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 16, 0xC1, 0x00, 0x99, 0x99, 0x00, 0x99, 0x99, 0x00, 0x64, 0x3F, 0x00, 0x25, 0x17, 0x00, 0x10, 0x0C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 16, 0xC2, 0x00, 0x99, 0x99, 0x00, 0x99, 0x99, 0x00, 0x64, 0x3F, 0x00, 0x25, 0x17, 0x00, 0x10, 0x0C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 16, 0xC3, 0x00, 0x99, 0x99, 0x00, 0x99, 0x99, 0x00, 0x64, 0x3F, 0x00, 0x25, 0x17, 0x00, 0x10, 0x0C);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 7, 0xC4, 0x22, 0x58, 0x58, 0x22, 0x58, 0x58);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 7, 0xC5, 0x00, 0x99, 0x99, 0x00, 0x99, 0x99);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 4, 0xC6, 0x00, 0xC8, 0xC8);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x0F);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0xF8, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x0A);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 8, 0xB2, 0x1F, 0x00, 0x00, 0x20, 0x01, 0x10, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 46, 0xB8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 43, 0xB9, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 43, 0xBA, 0x10, 0x10, 0x00, 0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x00, 0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x0B);
	hal_dsi_tx_ctrl_write_cmd(0x15, 0, 9, 0xB2, 0x01, 0x2F, 0x4F, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x15, 0, 54, 0xB9, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x70, 0x24, 0x0A, 0x6A, 0x13, 0x9A, 0xFA, 0x48, 0x14, 0xDA, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x15, 0, 54, 0xBA, 0x22, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x20, 0x20, 0x0A, 0x32, 0x00, 0x00, 0x00, 0x48, 0x38, 0xE8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x15, 0, 20, 0xC8, 0x0F, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x08, 0x08, 0x08, 0x08, 0x00, 0x80, 0x80, 0x80, 0x80);

//	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x04);
//	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0xB9, 0x91, 0x11);
	
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x48, 0x03);          // 0x00 command mode; 0x03 video mode
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0x51, 0x00, 0x00);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x53, 0xE8);
	hal_dsi_tx_ctrl_write_cmd(0x39, 0, 1, 0x35);
	
	
/*****FPS***/	
hal_dsi_tx_ctrl_write_cmd(0x39,0,2,0x9F,0x6);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xCD,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xCE,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,21,0xCF,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xD0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xD1,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,21,0xD2,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xD3,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xD4,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,21,0xD5,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xD6,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xD7,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,21,0xD8,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xD9,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xDA,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,21,0xDB,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xDC,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,41,0xDD,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,21,0xDE,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0,0x8,0x0);

///FPS BC CTRL///
hal_dsi_tx_ctrl_write_cmd(0x39,0,2,0x9F,0x5);
hal_dsi_tx_ctrl_write_cmd(0x39,0,13,0xB4,0x0,0x60,0xE,0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x22,0x8F); //HONOR

/// FPS CTRL ///
hal_dsi_tx_ctrl_write_cmd(0x39,0,2,0x9F,0x4);
hal_dsi_tx_ctrl_write_cmd(0x39,0,14,0xB5,0x11,0x17,0x26,0x0A,0x50,0x0,0x71,0x71,0x37,0x2A,0x71,0x2D,0xC1); ////R51 0F FF && Black AA pattern && Circle Lv. 1000nits  700 620 790
hal_dsi_tx_ctrl_write_cmd(0x39,0,18,0xE0,0x0,0x4F,0xFF,0xFF,0xFF,0x3F,0xFF,0xFF,0xFF,0x3F,0xFF,0xFF,0xFF,0x3F,0xFF,0xFF,0xFF);

//RF8 1E C0 C0 E0 00 00 00 00 E0
//RF9 E3 00 00 00 F0 C0 C0 00 00
//RFA 1E C0 C0 E0 00 00 00 00 E0

hal_dsi_tx_ctrl_write_cmd(0x39,0,10,0xF8,0x3E,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,10,0xF9,0xE3,0x0,0x0,0x0,0xC0,0x90,0x60,0x0,0x0);
hal_dsi_tx_ctrl_write_cmd(0x39,0,10,0xFA,0xFF,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0);

//FRC OFF FPS FIRST LINE
hal_dsi_tx_ctrl_write_cmd(0x39,0,2,0x9F,0x0A);
hal_dsi_tx_ctrl_write_cmd(0x39,0,2,0xED,0x00);
    //Bist mode
//    hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x01);
//    hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0xC6, 0x01, 0x11);        //循环：0x01, 0x11; 单色：0x03, 0x13
//    hal_dsi_tx_ctrl_write_cmd(0x39, 0, 2, 0x9F, 0x01);
//    hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0xC6, 0x03, 0x0F);        //循环：0x01, 0x11; 单色：0x03, 0x13
//    uint8_t read_array[1];
//    hal_dsi_tx_ctrl_read_cmd(0x06, 0, 0xDA, 1, read_array);
//    TAU_LOGD("0xDA: %02x\n", &read_array[1]);  
//	TAU_LOGD("Panel init!\n\r");
   // delayMs(180);
//	hal_dsi_tx_ctrl_write_cmd(0x05, 0, 1, 0x11);
	delayMs(60); //30
	
#endif
}

#if TX_CMD_MODE_WITHOUT_TE
/**
* @brief  MIPI RX事件处理函数,demo code 用于command mode 输出发送数据（模式1）
* @param  event:RX事件
* @param  data: user data
* @retval none
*/
static void app_rx_event_cb(hal_rx_event_e event, void *data)
{
    if (panel_display_done && event == HAL_RX_LINE_EVENT)
    {
        hal_dsi_tx_ctrl_gen_a_frame();
    }
}
#endif

#if TX_USE_CMD_MODE
/**
* @brief  TE引脚作为GPIO 输入回调函数,demo code 用于command mode 输出发送数据（模式2）
* @param  none
* @retval none
*/
static void app_tx_cmd_panel_te_cb(void *data)
{
    if (panel_display_done)
    {
        //delayUs(25); /* 撕裂调试 */
        hal_dsi_tx_ctrl_gen_a_frame();
    }
}

/**
* @brief  注册屏端TE信号输入引脚回调函数
* @param  pad :TE输入pad
* @param  trig:触发中断沿配置
* @retval none
*/
static void app_tx_cmd_app_init_panel_te_int(io_pad_e pad, sys_cfg_trigger_e trig)
{
    /*1.关闭中断*/
    hal_gpio_ctrl_eint(pad, DISABLE);

    /*2.中断初始化*/
    hal_gpio_init_eint(pad, trig);

    /*3.注册回调*/
    hal_gpio_reg_eint_cb(pad, app_tx_cmd_panel_te_cb);

    /*4.使能中断*/
    hal_gpio_ctrl_eint(pad, ENABLE);
}
#endif

/**
* @brief  mipi rx 初始化
* @param  none
* @retval none
*/
static void app_mipi_rx_init(void)
{
    if (g_rx_ctrl_handle == NULL)
    {
        /* 创建rx   ctrl handle */
        g_rx_ctrl_handle = hal_dsi_rx_ctrl_create_handle();
    }
    /* 配置参数 */
    g_rx_ctrl_handle->base_info.src_w = INPUT_WIDTH;
    g_rx_ctrl_handle->base_info.src_h = INPUT_HEIGHT;
    g_rx_ctrl_handle->base_info.dst_w = OUTPUT_WIDTH;
    g_rx_ctrl_handle->base_info.dst_h = OUTPUT_HEIGHT;
    g_rx_ctrl_handle->base_info.src_frate = INPUT_FRAME_RATE;
    g_rx_ctrl_handle->base_info.src_mode = INPUT_DATA_MODE;
    g_rx_ctrl_handle->base_info.dst_mode = OUTPUT_DATA_MODE;
    g_rx_ctrl_handle->rx_color_mode = INPUT_COLOR_MODE;
    g_rx_ctrl_handle->rx_lanes = INPUT_MIPI_LANE_NUM;
    g_rx_ctrl_handle->rx_nonburst_models = INPUT_VIDEO_MODEL; /* 可不配置 */
    g_rx_ctrl_handle->compress_en = INPUT_COMPRESS;
    g_rx_ctrl_handle->rx_hsclk_rate = INPUT_MIPI_LANE_RATE;
    g_rx_ctrl_handle->cus_dcs_entry_table = g_cus_rx_dcs_execute_table; /* 注册 DCS处理列表 */
    g_rx_ctrl_handle->rx_dcs_read_entry = ap_dcs_read; /* 注册dsc read 回调函数,可选,此函数为空时由cus_dcs_entry_table执行 */
	g_rx_ctrl_handle->hight_performan_mode=HIGHT_PERFORMAN_L2;

#if RX_RESOLUTION_CHANGE_ENABLE
    g_rx_ctrl_handle->pps_update_entry = pps_update_handle;
#endif
	g_rx_ctrl_handle->extra_info.crop_info.enable=1;
	g_rx_ctrl_handle->extra_info.crop_info.top =15;  //4 8 的倍数
	g_rx_ctrl_handle->extra_info.crop_info.right =2;  //4 8 的倍数


    /* 提前预置PPS,  AP 有PPS cmd也会更新 */
    if (g_rx_ctrl_handle->compress_en == true)
    {
        hal_dsi_rx_ctrl_pre_init_pps(g_rx_ctrl_handle, pps, 128);
    }


    /* 初始化rx ctrl */
    hal_dsi_rx_ctrl_init(g_rx_ctrl_handle);
	

#if RX_READ_HW_ACK
    /* 配置硬件回复 */
    app_set_dcs_hw_ack();
#endif

#if TX_CMD_MODE_WITHOUT_TE
    /* 注册接收一帧帧头事件回调,每接收一帧数据TX再往外发一帧 */
    //hal_dsi_rx_ctrl_register_callback(g_rx_ctrl_handle, app_rx_event_cb, HAL_RX_FS_EVENT, true, NULL);
    /* 注册接收第0行数据事件,接收到数据后再往外发送数据,确保不撕裂 */
    uint32_t line = 0;
    hal_dsi_rx_ctrl_register_callback(g_rx_ctrl_handle, app_rx_event_cb, HAL_RX_LINE_EVENT, true, &line);
#endif

#if RX_START_WITHOUT_RST
    /* 等待ap reset置位再启动rx,否则容易收到错误数据 */
    hal_dsi_rx_ctrl_start(g_rx_ctrl_handle);
#else
    /* 注册RX start callback,确认RX LP11时再启动RX,防止接收错误数据 */
    hal_gpio_set_ap_reset_int(ENABLE, app_mipi_rx_start_cb, DETECT_HIGH_LVL);
#endif
	TAU_LOGD("rx init!\n\r");
}

/**
* @brief  mipi tx 初始化
* @param  none
* @retval none
*/
static void app_mipi_tx_init(void)
{
    if (g_tx_ctrl_handle == NULL)
    {
        g_tx_ctrl_handle = hal_dsi_tx_ctrl_create_handle();
    }
    g_tx_ctrl_handle->channel_id = OUTPUT_VC;
    g_tx_ctrl_handle->lane_num = OUTPUT_LANE_NUMBER;
    g_tx_ctrl_handle->vid_mode = OUTPUT_VIDEO_MODEL;
    g_tx_ctrl_handle->cmd_tx_type = TX_INIT_TYPE;
    g_tx_ctrl_handle->dpi_vsa = OUTPUT_VSA;
    g_tx_ctrl_handle->dpi_vbp = OUTPUT_VBP;
    g_tx_ctrl_handle->dpi_vfp = OUTPUT_VFP;
    g_tx_ctrl_handle->dpi_hsa = OUTPUT_HSA;
    g_tx_ctrl_handle->dpi_hbp = OUTPUT_HBP;
    g_tx_ctrl_handle->dpi_hfp = OUTPUT_HFP;
    g_tx_ctrl_handle->base_info.src_w = INPUT_WIDTH;
    g_tx_ctrl_handle->base_info.src_h = INPUT_HEIGHT;
    g_tx_ctrl_handle->base_info.dst_w = OUTPUT_WIDTH;
    g_tx_ctrl_handle->base_info.dst_h = OUTPUT_HEIGHT;
    g_tx_ctrl_handle->base_info.src_frate = INPUT_FRAME_RATE;
    g_tx_ctrl_handle->base_info.src_mode = INPUT_DATA_MODE;
    g_tx_ctrl_handle->base_info.dst_mode = OUTPUT_DATA_MODE;
	g_tx_ctrl_handle->tx_frame_rate=58;

    hal_dsi_tx_ctrl_init(g_tx_ctrl_handle);
//	hal_dsi_tx_ctrl_set_cus_sync_line(g_tx_ctrl_handle,1200);  //1200

    /* FIXME set tear on*/
    hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_60HZ_MODE);

    /* AP 没有发送数据时默认的显示颜色, 量产为0 0 0(黑色), 配置其他颜色仅为debug使用 */
    hal_dsi_tx_ctrl_set_overwrite_rgb(0x00, 0x00, 0x00);
	TAU_LOGD("tx init!\n\r");
}

#if !RX_START_WITHOUT_RST
/**
* @brief  mipi rx start函数,开启AP RSTN等待启动配置后使用
* @param  none
* @retval none
*/
static void app_mipi_rx_start_cb(void *data)
{
    /* RX start */
    hal_dsi_rx_ctrl_start(g_rx_ctrl_handle);
    /* close cb */
    hal_gpio_set_ap_reset_int(DISABLE, NULL, DETECT_HIGH_LVL);
    TAU_LOGD("rx start\n");
}
#endif

void Panel_CCM(void)
{
	ccm_coef_t ccm;
	ccm.coef_c00 = 255;	// 260
	ccm.coef_c01 = 0;
	ccm.coef_c02 = 0;
	ccm.coef_c10 = 0;
	ccm.coef_c11 = 246;	// 247
	ccm.coef_c12 = 0;
	ccm.coef_c20 = 0;
	ccm.coef_c21 = 0;
	ccm.coef_c22 = 247;  //248

	hal_dsi_tx_ctrl_set_ccm(&ccm);
}

static void soft_swire_start(uint8_t pulse)
{
	do
	{
		hal_gpio_init_output( SWIRE_EN, IO_LVL_LOW  ); // 3us
		delayUs(10);
		hal_gpio_init_output( SWIRE_EN, IO_LVL_HIGH ); // 3us
		delayUs(10);
	}while(pulse--);
}

static void soft_swire_timer_callback(void *data)
{
	s_swire_pwm_counter++;
	if( s_swire_pwm_counter < SWIRE_MAX_NUM )
	{
		soft_swire_start(SWIRE_DEFAULT_NUM);
		hal_timer_start(SOFT_SWIRE_TIMER, 16, soft_swire_timer_callback, NULL);
	}
	else
	{
		hal_timer_deinit(SOFT_SWIRE_TIMER);
		hal_timer_stop(SOFT_SWIRE_TIMER);	
		hal_gpio_init_output(SWIRE_EN, IO_LVL_HIGH);
	}
}

static void soft_swire_init(void)
{
	hal_gpio_init_output(SWIRE_EN, IO_LVL_HIGH);
	delayMs(2);
	s_swire_pwm_counter = 0;
	soft_swire_start(SWIRE_DEFAULT_NUM);
	hal_timer_init(SOFT_SWIRE_TIMER);
    hal_timer_start(SOFT_SWIRE_TIMER, 16, soft_swire_timer_callback, NULL);
}


/**
* @brief  mipi tx 启动
* @param  none
* @retval none
*/
static void app_mipi_tx_start(void)
{
	/* Init panel */
	app_init_panel();
	hal_dsi_tx_ctrl_write_cmd(0x05, 0, 1, 0x11);
	delayMs(80); //30
	/* TX start */
	hal_dsi_tx_ctrl_start(g_tx_ctrl_handle);
    delayMs(50); //50
	hal_gpio_init_output(AVDD_EN, IO_LVL_HIGH);	
    delayMs(10); //10
	soft_swire_init();
//	hal_gpio_init_output(SWIRE_EN, IO_LVL_HIGH);
#if AP_SWIRE_OUTPUT
    hal_swire_enable(true);
    hal_swire_set_pulse(41);
#endif	
    panel_display_done = true;
    delayMs(50); //30	
    /* Display on */
    hal_dsi_tx_ctrl_write_cmd(0x05, 0, 1, 0x29);	
#if AP_SWIRE_OUTPUT
     hal_swire_set_pulse(SWIRE_DEFAULT_PULSE);
#endif
    TAU_LOGD("tx_start \n");
}

/**
* @brief  ap rstn 拉高中断回调,用于息屏唤醒
* @param  none
* @retval none
*/
static void ap_rstn_pull_high_cb(void *data)
{
    /* system resume begin */
    sg_system_resume = true;
    /* 关闭AP reset检查     */
    hal_gpio_set_ap_reset_int(DISABLE, NULL, DETECT_RISING_EDGE);
}

/**
* @brief  ap rstn 拉高中断回调,用于息屏待机
* @param  none
* @retval none
*/
static void ap_rstn_pull_down_cb(void *data)
{
	 TAU_LOGD("ap_rstn_pull_down_cb\n");
    sg_system_suspend = true;
    /* 关闭AP reset检查     */
    hal_gpio_set_ap_reset_int(DISABLE, NULL, DETECT_RISING_EDGE);
//    TAU_LOGD("ap_rstn_pull_down_cb\n");
}

/**
* @brief  GPIO初始化配置,根据实际原理图提前配置IO功能以及状态,默认功能可不配置
*         TP相关I2C/SPI 在tp_transfer.c,也可以挪到此函数初始化
* @param  none
* @retval none
*/
void app_gpio_init(void)
{
    io_pad_attr_t attrs[] =
    {
        {IO_PIN_8,  PIN8_MODE_GPIO7,     IO_IOE_OUTPUT, IO_LVL_LOW},/* PIN_8(TD_RSTN), GPIO,输出,低电平 */
        {IO_PIN_17, PIN17_MODE_GPIO8,   IO_IOE_OUTPUT, IO_LVL_LOW}, /* PIN_17(PWMEN), GPIO,输出,低电平 */
		 {IO_PIN_15, PIN15_MODE_GPIO4,   IO_IOE_OUTPUT, IO_LVL_LOW}, /* PIN_15(ASWIRE), GPIO,输出,低电平 */
   //     {IO_PIN_16, PIN16_MODE_GPIO2,   IO_IOE_OUTPUT, IO_LVL_LOW}, /* PIN_16(AP_INT),GPIO,输出,低电平 */
#if (TX_USE_CMD_MODE & (!TX_CMD_MODE_WITHOUT_TE))
        {IO_PIN_29, PIN29_MODE_GPIO3,   IO_IOE_INPUT, IO_LVL_NONE}, /* PIN_29(AP_TE), GPIO,输入 */
#else
        {IO_PIN_29, PIN29_MODE_TEAR,    IO_IOE_NONE, IO_LVL_NONE},  /* PIN_29(AP_TE), 硬件TEAR输出模式 */
#endif
    };
    uint8_t size = sizeof(attrs) / sizeof(io_pad_attr_t);
    hal_gpio_config_pad(attrs, size);
}


/**
* @brief  显示相关模块初始化,包括MIPI RX/TX/PWM/SWIRE/GPIO等
* @param  none
* @retval none
*/
void app_display_init(void)
{
    /* mipi rx初始化 */
    app_mipi_rx_init();
    /* VCC 主供电,等待VCC Power Ready,此时RX初始化完成可以响应MIPI命令 */
    if (MAIN_POWER_SELECT == PWR_SEL_VCC)
    {
		
        while (hal_pwr_get_vcc_power_ready() == false);
    }

    /* GPIO 初始化   */
    app_gpio_init();
//TAU_LOGD("app_gpio_init \n");
    /* 背光初始化      */
#if AP_SWIRE_OUTPUT
    hal_swire_init();   /*  swire init */
    hal_swire_set_timer(TIMER_NUM0, 17, true); /* swire连续发送,绑定timer进行发送 */
#endif
#if ANALOG_PWM_OUTPUT
    if (hal_pwm_init(PWM_FREQUENCY, PWM_DUTY_STEP))
    {
        hal_pwm_enable(true);
        hal_pwm_set_elvcc_output(true);
    }
#endif

    /* mipi tx 初始化*/
    app_mipi_tx_init();
//	soft_te_timer_init();
#if TX_START_AFTER_APRST
    TAU_LOGD("wait exit sleep mode\n");
#else
    app_mipi_tx_start();
#endif
}

/**
* @brief  系统resume
* @param  sleep_mode: sleep 模式
* @retval none
*/
static void app_system_resume(pwr_sleep_mode_e sleep_mode)
{
    /* 退出sleep mode, 电源切换 */
    hal_pwr_exit_sleep_mode();

    /* display resume */
    app_display_init();

#if TOUCH_ENABLE
    /* touch resume */
    app_tp_write_other_operations(NULL, 0);
#endif
    TAU_LOGD("system resume\n");
}

/**
* @brief  系统suspend,进入sleep mode
* @param  sleep_mode: sleep 模式
* @retval none
*/
static void app_system_suspend(pwr_sleep_mode_e sleep_mode)
{
//	TAU_LOGD("SLEEP_MODE\n");
    /* 关闭图像通路 */
    hal_dsi_tx_ctrl_stop(g_tx_ctrl_handle);
    hal_dsi_tx_ctrl_deinit(g_tx_ctrl_handle);
    hal_dsi_rx_ctrl_stop(g_rx_ctrl_handle);
    hal_dsi_rx_ctrl_deinit(g_rx_ctrl_handle);

    /* Tear拉低 */
    hal_gpio_init_output(IO_PAD_AP_TE, IO_LVL_LOW);
    panel_display_done = false;
#if RX_WAIT_TEAR_ON
    sg_ap_set_tear_on = false;
#endif

    /* 关闭外设 比如Swire/I2C/Flash 等 */
#if AP_SWIRE_OUTPUT
    hal_swire_deinit();
#endif
#if ANALOG_PWM_OUTPUT
    hal_pwm_deinit();
#endif

#if SHARE_FLASH_ENABLE
    hal_flash_share_mode(false);
#endif

    /* 切换TP18 供电 */
    hal_pwr_set_sleep_mode_power(SLEEP_MODE_POWER);

    if (sleep_mode == PWR_NORMAL_SLEEP_MODE)
    {
        /* normal sleep mode, MCU可以正常工作 */
        hal_gpio_set_ap_reset_int(ENABLE, ap_rstn_pull_high_cb, DETECT_HIGH_LVL);
        hal_pwr_enter_normal_sleep_mode();
		TAU_LOGD("PWR_NORMAL_SLEEP_MODE\n");
    }
    else if (sleep_mode == PWR_STOP_SLEEP_MODE)
    {
		TAU_LOGD("PWR_STOP_SLEEP_MODE\n");

        /* 注册对应 wakeup IO */
        hal_pwr_set_stop_sleep_wakeup_pin(IO_PAD_AP_RSTN, WUP_RISING_EDGE);
        //hal_pwr_set_stop_sleep_wakeup_pin(IO_PAD_AP_SPIS_CSN, WUP_FALLING_EDGE);
        //hal_pwr_set_stop_sleep_wakeup_pin(IO_PAD_TD_INT, WUP_FALLING_EDGE);
        io_pad_e wakeup_io = hal_pwr_enter_stop_sleep_mode();
        if (wakeup_io == IO_PAD_AP_RSTN)
        {
            sg_system_resume = true;
        }
        else
        {
            /* Not impletmented */
            TAU_LOGD("wakeup_io %d FIXME touch wakeup convert to AP\n", wakeup_io);
            hal_gpio_set_ap_reset_int(ENABLE, ap_rstn_pull_high_cb, DETECT_HIGH_LVL);
        }
    }
    else
    {
        /* deep sleep mode,配置AP RSTN 上升沿,TD INT 下降沿唤醒 */
        hal_pwr_enter_deep_sleep_mode(WUP_RISING_EDGE, WUP_FALLING_EDGE);
				TAU_LOGD("PWR_DEEP_SLEEP_MODE\n");
    }

}

/**
* @brief  系统process处理函数,处理待机唤醒等
* @param  none
* @retval none
*/
static void app_system_process(void)
{

    if (sg_system_suspend)
    {
		TAU_LOGD("app_system_process\n");
        /* 系统进入sleep mode */
        app_system_suspend(SLEEP_MODE_SELECT);
        sg_system_suspend = false;
    }

    if (sg_system_resume)
    {
        /* 系统退出sleep mode */
        app_system_resume(SLEEP_MODE_SELECT);
        sg_system_resume = false;
    }

#if TX_START_AFTER_APRST
    if (sg_tx_start_in_process)
    {
        app_mipi_tx_start();
        sg_tx_start_in_process = false;
    }
#endif
}



/**
* @brief  P6A demo 主函数
* @param  none
* @retval none
*/
void google_p6a_demo(void)
{
    /* 电源选择,上电只需要选择一次 */
    hal_pwr_set_main_power(MAIN_POWER_SELECT);        /* 切换供电*/
    /* 显示模块初始 */
    app_display_init();
    /* touch 相关模块初始化 */
#if TOUCH_ENABLE
    /* TP 初始化 */
    app_tp_init();
    app_tp_phone_clear_reset_on();
    /* 与屏的TP 模块通讯并初始化 */
    app_tp_transfer_screen_start();
#endif

    TAU_LOGD("p6a demo init done \n");

    while (1)
    {
		
		///hal_dsi_tx_ctrl_set_vpg(1, TX_VPG_H_COLOR, true);
#if TOUCH_ENABLE
        /* 等待屏TP中断上报做TP协议转换,注意接口实现不可阻塞!否则会影响sleep mode */
        app_tp_transfer_screen_int();
#endif
        /* DCS 命令异步处理 */
        while (hal_dsi_rx_ctrl_dcs_async_handler(g_rx_ctrl_handle));

        /* 系统事件处理(sleep mode) */
  //     app_system_process();
    }
}
#endif
